#include "elasttime.h"
#include "matrix.h"
#include "vector.h"
#include "stochdriver.h"
#include "global.h"
#include "intpoints.h"



/**
  Constructor initializes internal data prameters

  7.2008 TKo, TKr
*/
elasttime::elasttime (void)
{
  evf=b3law;
  e0 = 0.0;
  nu = 0.0;

  //b3:
  q1=q2=q3=q4=q5=0.0;
  type_e = 0;
  e28=30.0e9; //Pa
  fc=35.8e6; //Pa
  wc=0.43;
  sc=3.4;
  gc=1.98;
  cs=305.0;  //kg*m^-3
  a1=1.05;
  a2=1.2;
  kd=0.15;   //m
  tb_time = 0.0;//s 

  //double power law - je nutne opravit ??!!
  qs = 0.0;  psi = 0.0; n = 0.0; m = 0.0; alpha = 0.0;  
  t0 = 0.0;
}



elasttime::~elasttime (void)
{
}



/**
   Function reads material parameters.
   
   @param in - input stream

   7.2008 TKo, TKr
*/
void elasttime::read (XFILE *in)
{
  //xfscanf (in,"%lf",&nu);
  //xfscanf (in, "%ld",(int*)&evf);
  xfscanf (in,"%k%lf","nu",&nu);
  xfscanf (in, "%k%m", "ym_evolfunc", &ym_evolfunc_kwdset, &evf);
  switch (evf)
    {
    case b3law:
      xfscanf(in,"%ld",&type_e);
      
      if (type_e == 1){xfscanf (in,"%lf ",&e28);}
      //input in Pa
      e0 = e28;
      e28 = e28/6.89476*1.0e-3;//to psi
      
      xfscanf (in,"%lf %lf %lf %lf %lf %lf %lf %lf %lf %lf", &fc, &wc, &sc, &gc, &cs, &a1, &a2, &ks, &kd, &tb_time);
      
      //input in Pa
      fc = fc/6.89476*1.0e-3;//to psi
      //input in kg*m^-3
      cs = cs/16.03;//to lb*ft^-3
      //input in m
      kd=kd*1000.0/25.4;//to inch

      //pro vypocet zacinajici od casu 0.0
      tl = tb_time/86400;

      if(tl <= 0.0){
	print_err("Age of concrte is less than zero", __FILE__, __LINE__, __func__);
	exit(0);
      }
    
      break;
    case doublepwrlaw:
      xfscanf(in, "%k%lf %k%lf %k%lf %k%lf %k%lf %k%lf %k%lf", "e0", &e0, "t0", &t0, "qs", &qs, "psi", &psi, "m", &m, "alpha", &alpha, "n", &n);
      break;
    case userfunc:
      xfscanf(in, "%k%lf", "e0", &e0);
      gf.read(in);
      break;
    default:
      print_err("unknown type of function of Young modulus evolution", __FILE__, __LINE__, __func__);
  }
}



/**
  Function creates stiffness matrix depending on stress-strain state and 
  type of required stiffness matrix in problem description Mp->nlman.stmat 
  (initial, tangent,...).

  @param d    - stiffness matrix
  @param ssst - stress/strain state indicator

  7.2008 TKo
*/
void elasttime::matstiff (matrix &d,strastrestate ssst)
{
  switch (ssst)
  {
    case bar:
      matstiff_bar (d,secant_stiff);
      break;
    case plbeam:
      matstiff_plbeam (d,secant_stiff);
      break;
    case spacebeam:
      matstiff_spacebeam (d,secant_stiff);
      break;
    case planestress:
     matstiff_plstress (d,secant_stiff);
     break;
    case planestrain:
      matstiff_plstrain (d,secant_stiff);
      break;
    case platek:
      matstiff_plate (d,secant_stiff);
      break;
    case plates:
      matstiff_plate (d,secant_stiff);
      break;
    case axisymm:
      matstiff_axi (d,secant_stiff);
      break;
    case spacestress:
      matstiff_spacestr (d,secant_stiff);
      break;
    default:
      print_err("unknown number of components of stress tensor is required",__FILE__,__LINE__,__func__);
  }
}



/**
  Function creates stiffness matrix depending on stress-strain state and 
  type of required stiffness matrix (initial, tangent,...) given by parameter
  smt.

  @param d    - stiffness matrix
  @param ssst - stress/strain state indicator
  @param smt  - matrix type indicator

  7.2008 TKo
*/
void elasttime::matstiff (matrix &d,strastrestate ssst, stiffmatrix smt)
{
  switch (ssst)
  {
    case bar:
      matstiff_bar (d,smt);
      break;
    case plbeam:
      matstiff_plbeam (d,smt);
      break;
    case spacebeam:
      matstiff_spacebeam (d,smt);
      break;
    case planestress:
     matstiff_plstress (d,smt);
     break;
    case planestrain:
      matstiff_plstrain (d,smt);
      break;
    case platek:
      matstiff_plate (d,smt);
      break;
    case plates:
      matstiff_plate (d,smt);
      break;
    case axisymm:
      matstiff_axi (d,smt);
      break;
    case spacestress:
      matstiff_spacestr (d,smt);
      break;
    default:
      print_err("unknown number of components of stress tensor is required",__FILE__,__LINE__,__func__);
  }
}



/**
  Function creates intial stiffness matrix depending on stress-strain state.

  @param d    - stiffness matrix
  @param ssst - stress/strain state indicator

  7.2008 TKo
*/
void elasttime::elmatstiff (matrix &d,strastrestate ssst)
{
  switch (ssst){
  case bar:{
    matstiff_bar (d,initial_stiff);
    break;
  }
  case plbeam:{
    matstiff_plbeam (d,initial_stiff);
    break;
  }
  case spacebeam:{
    matstiff_spacebeam (d,initial_stiff);
    break;
  }
  case planestress:{
    matstiff_plstress (d,initial_stiff);
    break;
  }
  case planestrain:{
    matstiff_plstrain (d,initial_stiff);
    break;
  }
  case platek:{
    matstiff_plate (d,initial_stiff);
    break;
  }
  case plates:{
    matstiff_plate (d,initial_stiff);
    break;
  }
  case axisymm:{
    matstiff_axi (d,initial_stiff);
    break;
  }
  case spacestress:{
    matstiff_spacestr (d,initial_stiff);
    break;
  }
  default:{
    print_err("unknown number of components of stress tensor is required",__FILE__,__LINE__,__func__);
  }
  }
}



/**
   Function creates stiffness matrix for bar elements.
   
   @param d   - stiffness matrix of the material
   @param smt - type of assembled stiffness matrix (initial, tangent, ...)
   
   TKo 7.2008
*/
void elasttime::matstiff_bar (matrix &d, stiffmatrix smt)
{
  /*
  if (smt == initial_stiff)
    e = e0;
  else
    e = actual_modulus();
  */
  double e;
  if (smt == initial_stiff)
    e = e0;
  else
    e = actual_modulus();

  d[0][0] = e;
}



/**
   Function creates stiffness matrix for plane beam elements.
   
   @param d   - stiffness matrix of the material
   @param smt - type of assembled stiffness matrix (initial, tangent, ...)
   
   TKo 7.2008
*/
void elasttime::matstiff_plbeam (matrix &d, stiffmatrix smt)
{
  double e;
  if (smt == initial_stiff)
    e = e0;
  else
    e = actual_modulus();

  d[0][0] = e;
  d[1][1] = e/2.0/(1.0+nu);
  d[2][2] = e;
}



/**
   Function creates stiffness matrix for space beam elements.
   
   @param d   - stiffness matrix of the material
   @param smt - type of assembled stiffness matrix (initial, tangent, ...)
   
   TKo 7.2008
*/
void elasttime::matstiff_spacebeam (matrix &d, stiffmatrix smt)
{
  double e;
  if (smt == initial_stiff)
    e = e0;
  else
    e = actual_modulus();

  d[0][0] = e;
  d[1][1] = e/2.0/(1.0+nu);
  d[2][2] = e/2.0/(1.0+nu);
  d[3][3] = e/2.0/(1.0+nu);
  d[4][4] = e;
  d[5][5] = e;
}



/**
   Function creates stiffness matrix for 2D problems (plane stress).

   @param d   - stiffness matrix of the material
   @param smt - type of assembled stiffness matrix (initial, tangent, ...)
   
   TKo 7.2008
*/
void elasttime::matstiff_plstress (matrix &d, stiffmatrix smt)
{
  double c;
  double e;
  if (smt == initial_stiff)
    e = e0;
  else
    e = actual_modulus();

  
  fillm(0.0,d);

  c = e/(1.0-nu*nu);
  
  d[0][0] = c;     d[0][1] = c*nu;  d[0][2] = 0.0;
  d[1][0] = c*nu;  d[1][1] = c;     d[1][2] = 0.0;
  d[2][0] = 0.0;   d[2][1] = 0.0;   d[2][2] = e/2.0/(1.0+nu);
}



/**
   Function creates stiffness matrix for 2D problems (plane strain).
 
   @param d   - stiffness matrix of the material
   @param smt - type of assembled stiffness matrix (initial, tangent, ...)
   
   TKo 7.2008
*/
void elasttime::matstiff_plstrain (matrix &d, stiffmatrix smt)
{
  double c;
  double e;
  if (smt == initial_stiff)
    e = e0;
  else
    e = actual_modulus();

  
  fillm(0.0,d);

  c = e/(1.0+nu)/(1.0-2.0*nu);
  
  d[0][0] = c*(1.0-nu);   d[0][1] = c*nu;         d[0][2] = 0.0;
  d[1][0] = c*nu;         d[1][1] = c*(1.0-nu);   d[1][2] = 0.0;
  d[2][0] = 0.0;          d[2][1] = 0.0;          d[2][2] = e/2.0/(1.0+nu);

  if (d.m > 3)
    {
      d[0][3] = d[0][1]; d[1][3] = d[1][0];
      d[3][0] = d[1][0]; d[3][1] = d[1][0]; d[3][3] = d[1][1];
    }
}



/**
   Function creates stiffness matrix for 2D problems (axisymmetric problem).

   @param d   - stiffness matrix of the material
   @param smt - type of assembled stiffness matrix (initial, tangent, ...)
   
   TKo 7.2008
*/
void elasttime::matstiff_axi (matrix &d, stiffmatrix smt)
{
  double g,s;
  double e;
  if (smt == initial_stiff)
    e = e0;
  else
    e = actual_modulus();

  
  fillm(0.0,d);
  
  g = e/2.0/(1.0+nu);
  s = e/(1.0+nu)/(1.0-2.0*nu);
  
  d[0][0]=s*(1-nu);  d[0][1]=s*nu;     d[0][2]=d[0][1];
  d[1][0]=d[0][1];   d[1][1]=d[0][0];  d[1][2]=d[0][1];
  d[2][0]=d[0][1];   d[2][1]=d[0][1];  d[2][2]=d[0][0];

  d[3][3]=g;

}



/**
   Function creates stiffness matrix for plate elements.
   
   @param d   - stiffness matrix of the material
   @param smt - type of assembled stiffness matrix (initial, tangent, ...)
   
   TKo 7.2008
*/
void elasttime::matstiff_plate (matrix &d, stiffmatrix smt)
{
  double c,g;
  double e;
  if (smt == initial_stiff)
    e = e0;
  else
    e = actual_modulus();

  
  fillm(0.0,d);

  c = e/12.0/(1.0-nu*nu);
  g = e/2.0/(1.0+nu);
  
  d[0][0]=c;        d[0][1]=c*nu;  d[0][2]=0.0;
  d[1][0]=d[0][1];  d[1][1]=c;     d[1][2]=0.0;
  d[2][0]=0.0;      d[2][1]=0.0;   d[2][2]=g/12.0;
  
  d[3][3]=g;  d[4][4]=g;
}



/**
   Function creates stiffness matrix for 3D problems.
   
   @param d   - stiffness matrix of the material
   @param smt - type of assembled stiffness matrix (initial, tangent, ...)
   
   TKo 7.2008
*/
void elasttime::matstiff_spacestr (matrix &d, stiffmatrix smt)
{
  double g,s;
  double e;
  if (smt == initial_stiff)
    e = e0;
  else
    e = actual_modulus();

  
  fillm(0.0,d);
  
  g = e/2.0/(1.0+nu);
  s = e/(1.0+nu)/(1.0-2.0*nu);
  
  d[0][0]=s*(1-nu);  d[0][1]=s*nu;     d[0][2]=s*nu;
  d[1][0]=d[0][1];   d[1][1]=d[0][0];  d[1][2]=d[0][1];
  d[2][0]=d[0][1];   d[2][1]=d[0][1];  d[2][2]=d[0][0];

  d[3][3]=g;         d[4][4]=g;        d[5][5]=g;
}



/**
  Function creates complience matrix depending on stress-strain state and 
  type of required stiffness matrix in problem description Mp->nlman.stmat (initial, tangent,...).

  @param c    - compliance matrix of the material
  @param ssst - stress/strain state indicator

  7.2008 TKo
*/
void elasttime::matcompl (matrix &c,strastrestate ssst)
{
  switch (ssst)
  {
    case bar:
      matcompl_bar (c,Mp->nlman.stmat);
      break;
    case plbeam:
      matcompl_plbeam (c,Mp->nlman.stmat);
      break;
    case planestress:
      matcompl_plstress (c,Mp->nlman.stmat);
      break;
    case planestrain:
      matcompl_plstrain (c,Mp->nlman.stmat);
      break;
    case axisymm:
      matcompl_axi (c,Mp->nlman.stmat);
      break;
    case spacestress:
      matcompl_spacestr (c,Mp->nlman.stmat);
      break;
    default:
      print_err ("unknown number of components of stress tensor is required",__FILE__,__LINE__,__func__);
  }
}



/**
   Function creates compliance matrix for bar elements.
   
   @param c   - compliance matrix of the material
   @param smt - type of assembled stiffness matrix (initial, tangent, ...)
   
   7.2008 TKo
*/
void elasttime::matcompl_bar (matrix &c, stiffmatrix smt)
{
  double e;
  if (smt == initial_stiff)
    e = e0;
  else
    e = actual_modulus();

  c[0][0] = 1.0/e;
}



/**
   Function creates compliance matrix for plane beam elements.
   
   @param c   - compliance matrix of the material
   @param smt - type of assembled stiffness matrix (initial, tangent, ...)
   
   7.2008 TKo
*/
void elasttime::matcompl_plbeam (matrix &c, stiffmatrix smt)
{
  double e;
  if (smt == initial_stiff)
    e = e0;
  else
    e = actual_modulus();

  c[0][0] = 1.0/e;
  c[1][1] = 2.0*(1.0+nu)/e;
  c[2][2] = 1.0/e;
}



/**
   Function creates compliance matrix for 2D problems (plane stress).

   @param c   - compliance matrix of the material
   @param smt - type of assembled stiffness matrix (initial, tangent, ...)
   
   7.2008 TKo
*/
void elasttime::matcompl_plstress (matrix &c, stiffmatrix smt)
{
  double e;
  if (smt == initial_stiff)
    e = e0;
  else
    e = actual_modulus();

  fillm(0.0,c);
  
  c[0][0] =  1.0/e;     c[0][1] = -1.0*nu/e;  c[0][2] = 0.0;
  c[1][0] = -1.0*nu/e;  c[1][1] =  1.0/e;     c[1][2] = 0.0;
  c[2][0] = 0.0;        c[2][1] = 0.0;        c[2][2] = 2.0*(1.0+nu)/e;
}



/**
   Function creates compliance matrix for 2D problems (plane strain).

   @param c   - compliance matrix of the material
   @param smt - type of assembled stiffness matrix (initial, tangent, ...)
   
   7.2008 TKo
*/
void elasttime::matcompl_plstrain (matrix &c, stiffmatrix smt)
{
  double g;
  double e;
  if (smt == initial_stiff)
    e = e0;
  else
    e = actual_modulus();

  
  fillm(0.0,c);
  
  g = (1.0+nu)/e;
  
  c[0][0] = g*(1.0-nu);   c[0][1] = -1.0*g*nu;    c[0][2] = 0.0;
  c[1][0] = -1.0*g*nu;    c[1][1] = g*(1.0-nu);   c[1][2] = 0.0;
  c[2][0] = 0.0;          c[2][1] = 0.0;          c[2][2] = 2.0*g;
}



/**
   Function creates compliance matrix for axisymmetric problems
   
   @param c   - compliance matrix of the material
   @param smt - type of assembled stiffness matrix (initial, tangent, ...)
   
   7.2008 TKo
*/
void elasttime::matcompl_axi (matrix &c, stiffmatrix smt)
{
  double g;
  double e;
  if (smt == initial_stiff)
    e = e0;
  else
    e = actual_modulus();

  fillm(0.0,c);
  
  g = 2.0*(1.0+nu)/e;
  
  c[0][0]=1.0/e;     c[0][1]=-1.0*nu/e;  c[0][2]=c[0][1];    c[0][3]=0.0;
  c[1][0]=c[0][1];   c[1][1]=c[0][0];    c[1][2]=c[0][1];    c[1][3]=0.0;
  c[2][0]=c[0][1];   c[2][1]=c[0][1];    c[2][2]=c[0][0];    c[2][3]=0.0;

  c[3][0]=c[0][3];   c[3][1]=c[1][3];    c[3][2]=c[2][3];    c[3][3]=g;
}



/**
   Function creates compliance matrix for 3D problems.
   
   @param c   - compliance matrix of the material
   @param smt - type of assembled stiffness matrix (initial, tangent, ...)
   
   7.2008 TKo
*/
void elasttime::matcompl_spacestr (matrix &c, stiffmatrix smt)
{
  double g;
  double e;
  if (smt == initial_stiff)
    e = e0;
  else
    e = actual_modulus();

  fillm(0.0,c);
  
  g = 2.0*(1.0+nu)/e;
  
  c[0][0]=1.0/e;     c[0][1]=-1.0*nu/e;  c[0][2]=c[0][1];
  c[1][0]=c[0][1];   c[1][1]=c[0][0];    c[1][2]=c[0][1];
  c[2][0]=c[0][1];   c[2][1]=c[0][1];    c[2][2]=c[0][0];

  c[3][3]=g;         c[4][4]=g;          c[5][5]=g;
}



/**
  Function computes value of effective Young modulus from given time.
 
  @retval actual value of Young modulus
  TKo 7.2008
*/
double elasttime::actual_modulus()
{
  double e, j, tmp;
  double c0,ac,ag,m,z,r,qf,q;
  double t;

  switch (evf)
  {
  case b3law://b3 model for concrete - basic creep       
    //pro vypocet zacinajici od casu 0.0
    t = tb_time/86400.0 + Mp->time/86400;

    ac=sc+gc;   ag=ac/gc;   m=0.5; //m=0.28+1.0/fc/fc;
    n = 0.1;
    
    if (type_e == 1)
      q1=600000.0/e28;//measured
    else
      q1=600000.0/57000.0/sqrt(fc);//empirical
    
    r=1.7*pow(tl,0.12)+8.0;
    qf=1.0/(0.086*pow(tl,(2.0/9.0))+1.21*pow(tl,(4.0/9.0)));
    z = 1.0/pow(tl,m)*log(1.0 + pow((t - tl),n));
    q = qf/pow((1.0 + pow((qf/z),r)),(1.0/r));
    q2=451.1*sqrt(cs)/pow(fc,0.9);
    q3=0.29*pow(wc,4.0)*q2;
    q4=0.14/pow(ac,0.7);
    
    c0=q2*q+q3*log(1.0+pow((t-tl),n))+q4*log(t/tl);

    j = (q1 + c0)/6.89476*1.0e-3*1.0e-6;//units 1e-6*psi^-1 -> Pa^-1

    e = 1.0/j; //effective modulus
    if (e0 == 0.0)
      e0 = e;
    break;
  case doublepwrlaw://toto neni double power law - je nutne opravit
    //pro vypocet zacinajici od casu 0.0
    t = tb_time/86400.0 + Mp->time/86400;
    m=0.5;
    e=e0;
    tmp = 1+psi*(pow(t0,-m)+alpha)*pow(t,n);
    j = 1/e0 + qs*log(tmp);
    e = 1.0/j;
    break;
  case userfunc:
    gf.getval(Mp->time);
  default:
    print_err("unknown type of function of Young modulus evolution", __FILE__, __LINE__, __func__);
  }
  return e;
}



/**
  Function returns value of initial Young modulus.
 
  @retval actual value of Young modulus

  TKo 7.2008
*/
double elasttime::initial_modulus()
{
  return e0;
}



/**
   function computes true stresses
   
   @param ipp - number of integration point
   
*/
void elasttime::nlstresses (long ipp)
{
  long i, n = Mm->ip[ipp].ncompstr;
  vector eps(n),sig(n);
  strastrestate ssst = Mm->ip[ipp].ssst;
  matrix d(n,n);
  
  //  initial values
  for (i=0;i<n;i++){
    eps[i]=Mm->ip[ipp].strain[i];
  }
  
  matstiff (d,ssst,secant_stiff);
  mxv (d,eps,sig);
  if (Mm->ip[ipp].ssst == planestress)
    Mm->ip[ipp].strain[3] = -nu / (1.0 - nu) * (eps[0]+eps[1]);

  for (i=0;i<n;i++){
    Mm->ip[ipp].stress[i]=sig[i];
  }
  
}

void elasttime::changeparam (atsel &atm,vector &val)
{
  long i;
  
  for (i=0;i<atm.num;i++)
  {
    switch (atm.atrib[i])
    {
      case 0:
        e0=val[i];
        break;
      case 1:
        nu=val[i];
        break;
      case 2:
        qs = val[i];  
        break;
      case 3:
       psi = val[i]; 
        break;
      case 4:
        n = val[i]; 
        break;
      case 5:
        m = val[i];  
        break;
      case 6:
        alpha = val[i];  
        break;
      default:
        print_err("wrong number of atribute in function changeparam",__FILE__,__LINE__, __func__);
    }
  }
}
