#include <math.h>
#include <string.h>
#include <stdlib.h>
#include "nssolver.h"
#include "slopesol.h"
#include "global.h"
#include "globmat.h"
#include "mechprint.h"
#include "gmatrix.h"
#include "gtopology.h"
#include "loadcase.h"
#include "dloadcase.h"
#include "intpoints.h"
#include "auxdatout.h"
#include "mechprint.h"
#include "j2flow.h" // pak vyhodit
#include "chen.h" // pak vyhodit
#include "hardsoft.h" // pak vyhodit
#include "mathem.h"
#include "vector.h"
#include "mtsolver.h"

#include "adaptivity.h"
#include "meshtransfer.h"
#include "elemswitch.h"



/**
  Function solves non-linear statics problems.
   
  Created by JK,
*/
void solve_nonlinear_statics ()
{
  long i;
  double *rhs;
  
  if (Mp->nlman.tnlinsol==arcl)
    seldofinit ();
  
  Mp->lambda = 1.0;  
  rhs=Lsrs->give_rhs (0);
  
  for (i=0;i<Lsrs->nlc;i++){
    
    //  vectors of loading
    mefel_right_hand_side (i,rhs);
    
    if (!Mp->adaptivity)
      nonlinear_solver (i);
    else
      nonlinear_solver_adaptiv (i);
    
  }
  
}



/**
  Function calls selected solver of systems of nonlinear equation 
  for the given load case.

  @param lcid - load case id

  @return The function does not return anything.

  Created by Tomas Koudelka,
*/
void nonlinear_solver (long lcid)
{
  switch (Mp->nlman.tnlinsol){
  case arcl:{
    arclength (lcid);
    break;
  }
  case newton:{
    newton_raphson (lcid);
    break;
  }
  case arclrv1:{
    arclengthrv1 (lcid,1.1,1.0e-3);
    break;
  }
  case newtonrv1:{
    newton_raphsonrv1 (lcid,Mp->nlman.rvlam,1.0e-3);
    break;
  }
  case newtonrestart:{
    newton_raphson_restart (lcid);
    break;
  }
  case displctrl:{
    displ_control (lcid);
    break;
  }
  case displctrlrv:{
    displ_controlrv (lcid, 0.0, 1.0, Mp->nlman.rvlam, Mp->nlman.errl, 1);
    break;
  }
  case arclrv:{
    arclengthrv1 (lcid, Mp->nlman.rvlam, Mp->nlman.errl);
    break;
  }
  case adaptram:{
    arclengthadapt (lcid,0);
    break;
  }
  default:
    print_err("unknown solver of nonlinear equations system is required", __FILE__, __LINE__, __func__);
  }
}



/**
  Function solves system of nonlinear algebraic equations by
  Newton-Raphson method for the given load case.
  Solved system does not contain time variable.
  
  @param lcid - load case id

  @return The function does not return anything.

  Created by JK,  16.8.2001
*/
void newton_raphson (long lcid)
{
  long i,j,k,n,ni,ini,modif;
  double lambda,blambda,dlambda,dlambdamin,dlambdamax,zero,err,norf,norfa;
  double *r,*rb,*dr,*f,*fi,*fb,*fc,*fp;
  matrix lsm_a(3,3);
  vector lsm_r(3), lsm_l(3);


  //  number of rows of the matrix
  n = Ndofm;
  //  maximum number of increments
  ni = Mp->nlman.ninr;
  //  maximum number of iterations in one increment
  ini = Mp->nlman.niilnr;
  //  computer zero
  zero = Mp->zero;
  //  required error in inner loop
  err = Mp->nlman.errnr;
  //  increment size
  dlambda=Mp->nlman.incrnr;
  //  minimum increment size
  dlambdamin=Mp->nlman.minincrnr;
  //  maximum increment size
  dlambdamax=Mp->nlman.maxincrnr;

  rb = new double [n];
  f  = new double [n];
  fb = new double [n];
  fi = new double [n];
  dr = new double [n];
  memset (rb,0,n*sizeof(double));
  memset (f,0,n*sizeof(double));
  memset (fi,0,n*sizeof(double));
  memset (dr,0,n*sizeof(double));
  
  //  initialization phase
  r  = Lsrs->give_lhs (lcid);
  fp = Lsrs->give_rhs (lcid*2);
  fc = Lsrs->give_rhs (lcid*2+1);
  lambda=0.0;
  
  modif=0;
  
  // ***************************
  //  main iteration loop  ****
  // ***************************
  print_init(-1, "wt");
  print_step(lcid, 0, 0.0, f);
  print_flush();


  for (i=0;i<ni;i++){
    
    //  indicator of strain computation
    //  it means, strains at integration points have not been computed
    Mp->strainstate=0;
    //  indicator of stress computation
    //  it means, stresses at integration points have not been computed
    Mp->stressstate=0;
    //  indicator of computation of other array
    //  it means, stresses at integration points have not been computed
    Mp->otherstate=0;


    //  backup of left hand side vector
    for (j=0;j<n;j++){
      rb[j]=r[j];
    }
    //  backup of reached lambda parameter
    blambda=lambda;
   
    fprintf (stdout,"\n increment %ld   lambda %e,    dlambda %e",i,lambda,dlambda);
    
    //  vector of maximum load and vector of load increment
    for (j=0;j<n;j++){
      f[j]=fc[j]+lambda*fp[j];
      fb[j]=dlambda*fp[j];
    }
    
    //  assembling of tangent stiffness matrix
    if ((Mp->nlman.stmat==tangent_stiff) || (i == 0))
      stiffness_matrix (lcid);
    
    //  solution of K(r).v=F
    //Smat->solve_system (Gtm,dr,fb);
    Mp->ssle->solve_system (Gtm,Smat,dr,fb,Out);

    for (j=0;j<n;j++){
      r[j]+=dr[j];
    }
    
    //  computation of internal forces
    internal_forces (lcid,fi);
    
    //  vector of unbalanced forces
    for (j=0;j<n;j++){
      fb[j]=f[j]+dlambda*fp[j];
    }
    norfa=sizev(fb,n);
    for (j=0;j<n;j++){
      fb[j] -= fi[j];
    }
    //  norm of vector of unbalanced forces
    norf=sizev(fb,n);
    norf /= norfa;
    
//    fprintf (Out,"\n norf=%e",norf);

    if (norf<err){

      lambda+=dlambda;
      Mm->updateipval ();
      compute_req_val (lcid);
      print_step(lcid, i+1, lambda, f);
      print_flush();

      
      modif++;
      if (modif>1){
	dlambda*=2.0;
	if (dlambda>dlambdamax)
	  dlambda=dlambdamax;

	if (Mespr==1){
	  fprintf (stdout,"\n increment must be modified (dlambda increase) dlambda=%e",dlambda);
	}
	modif=0;
      }
      
      continue;
    }
    
    //  internal iteration loop
    fillm(0.0, lsm_a);
    fillv(0.0, lsm_r);
    for (j=0;j<ini;j++){
      
      //  solution of K(r).v=F
      //Smat->solve_system (Gtm,dr,fb);
      Mp->ssle->solve_system (Gtm,Smat,dr,fb,Out);
    
      for (k=0;k<n;k++){
	r[k]+=dr[k];
      }
      
      //  computation of internal forces
      internal_forces (lcid,fi);
      
      //  vector of unbalanced forces
      for (k=0;k<n;k++){
	fb[k]=f[k]+dlambda*fp[k]-fi[k];
      }
      
      //  norm of vector of unbalanced forces
      norf=sizev(fb,n);
      norf /= norfa;
      
      fprintf (stdout,"\n increment %ld   inner loop j %ld     norf=%e  dlambda %e",i,j,norf,dlambda);
//      fprintf (Out,"\n j=%ld     norf=%e",j,norf);
      
      if (norf<err)  break;

      // divergence detection with help of least square method
      if (j > 1)
      {
        if (lsm_quad(lsm_a, lsm_r, lsm_l, double(j+1), norf, Mp->zero,1) > 0.0)
        {
	  fprintf (stdout,"\n\n Divergence of inner iteation loop is detected. Inner loop is stopped.\n\n");
	  break;
	}
      }
      else
        lsm_quad(lsm_a, lsm_r, lsm_l, double(j+1), norf, Mp->zero,0);
    }
    
    modif=0;

    if (j==ini || norf>err){
      dlambda/=2.0;
      if (dlambda<dlambdamin){
	dlambda=dlambdamin;
	break;
      }

      if (Mespr==1){
	fprintf (stdout,"\n increment must be modified (dlambda decrease) dlambda=%e    norf=%e",dlambda,norf);
//	fprintf (Out,"\n increment must be modified (dlambda decrease) dlambda=%e    norf=%e",dlambda,norf);
      }
      
      for (j=0;j<n;j++){
      	r[j]=rb[j];
      }
      lambda=blambda;
    }
    else{
      lambda+=dlambda;
      Mm->updateipval ();
      compute_req_val (lcid);
      print_step(lcid, i+1, lambda, f);
      print_flush();
    }
    
  }
  
  delete [] dr;  delete [] fi;  delete [] fb;  delete [] f;
  print_close();
}



/**
  Function solves system of nonlinear algebraic equations by 
  Newton-Raphson method for the given load case. Solved system
  does not contain time variable and direct displacement control 
  is assumed.
  
  @param lcid - load case id

  @return The function does not return anything.

  Created by Tomas Koudelka, 2.11.2005
*/
void displ_control (long lcid)
{
  long i,j,k,n,ni,ini,incs;
  double lambda,blambda,dlambda,dlambdamin,zero,err,norf,norfa, dlambdamax;
  double *r,*rb,*dr,*f,*fi,*fb,*fc,*fp, *fir2;
  matrix lsm_a(3,3);
  vector lsm_r(3), lsm_l(3);


  //  number of rows of the matrix
  n = Ndofm;
  //  maximum number of increments
  ni = Mp->nlman.ninr;
  //  maximum number of iterations in one increment
  ini = Mp->nlman.niilnr;
  //  computer zero
  zero = Mp->zero;
  //  required error in inner loop
  err = Mp->nlman.errnr;
  //  increment size
  dlambda=Mp->nlman.incrnr;
  //  minimum increment size
  dlambdamin=Mp->nlman.minincrnr;
  // maximum increment size
  dlambdamax=Mp->nlman.incrnr;
 
  rb = new double [n];
  f  = new double [n];
  fb = new double [n];
  fi = new double [n];
  fir2 = new double[n];
  dr = new double [n];
  memset (rb,0,n*sizeof(double));
  memset (f,0,n*sizeof(double));
  memset (fi,0,n*sizeof(double));
  memset (dr,0,n*sizeof(double));
  
  //  initialization phase
  r  = Lsrs->give_lhs (lcid);
  fp = Lsrs->give_rhs (lcid*2);
  fc = Lsrs->give_rhs (lcid*2+1);
  lambda=0.0;
  Mp->lambda = lambda;
  
  // ***************************
  //  main iteration loop  ****
  // ***************************
  print_init(-1, "wt");
  print_step(lcid, 0, 0.0, f);
  print_flush();
  incs = 0;
  for (i=0;i<ni;i++){
    Mp->istep = i;
    Mp->jstep = -1;
    //  backup of left hand side vector
    for (j=0;j<n;j++){
      rb[j]=r[j];
    }
    //  backup of reached lambda parameter
    blambda=lambda;
    Mp->lambda = lambda;
   
    fprintf (stdout,"\n increment %ld   lambda %e,    dlambda %e",i,lambda,dlambda);
    
    //  vector of maximum load and vector of load increment
    Mp->lambda += dlambda;
    internal_forces(lcid, fir2);
    Mp->jstep = 0;

    for (j=0;j<n;j++){
      f[j]=fc[j]+lambda*fp[j];
      fb[j]= f[j] + dlambda*fp[j] - fir2[j];
    }
    
    //  assembling of tangent stiffness matrix
    if ((Mp->nlman.stmat==tangent_stiff) || (i == 0))
      stiffness_matrix (lcid);
    
    //  solution of K(r).v=F
    //Smat->solve_system (Gtm,dr,fb);
    Mp->ssle->solve_system (Gtm,Smat,dr,fb,Out);

    for (j=0;j<n;j++){
      r[j]+=dr[j];
    }
    
    //  computation of internal forces
    internal_forces (lcid,fi);
    
    //  vector of unbalanced forces
    for (j=0;j<n;j++){
      fb[j]=f[j]+dlambda*fp[j];
    }
    norfa=sizev(fb,n);
    for (j=0;j<n;j++){
      fb[j] -= fi[j];
    }
    //  norm of vector of unbalanced forces
    norf=sizev(fb,n);
    if (norfa != 0.0)
      norf /= norfa;
    
    fprintf (stdout,",    norm %e",norf);
    if (norf<err){
      lambda+=dlambda;
      Mm->updateipval ();
      compute_req_val (lcid);
      print_step(lcid, i+1, lambda, f);
      print_flush();
      continue;
    }
    
    fillm(0.0, lsm_a);
    fillv(0.0, lsm_r);
    stiffness_matrix (lcid);
    //  internal iteration loop
    for (j=0;j<ini;j++){
      Mp->jstep = j;      
      //  solution of K(r).v=F
      //Smat->solve_system (Gtm,dr,fb);
      Mp->ssle->solve_system (Gtm,Smat,dr,fb,Out);

      for (k=0;k<n;k++){
	r[k]+=dr[k];
      }
      
      //  computation of internal forces
      internal_forces (lcid,fi);
      
      //  vector of unbalanced forces
      for (k=0;k<n;k++){
	fb[k]=f[k]+dlambda*fp[k]-fi[k];
      }
      
      //  norm of vector of unbalanced forces
      norf=sizev(fb,n);
      if (norfa != 0.0)
        norf /= norfa;
      
      fprintf (stdout,"\n increment %ld   inner loop j %ld     norf=%e  dlambda %e",i,j,norf,dlambda);
      
      if (norf<err)  
        break;
      // divergence detection with help of least square method
/**      if (j > 1)
      {
        if (lsm_quad(lsm_a, lsm_r, lsm_l, double(j+1), norf, Mp->zero,1) > 0.0)
        {
	  fprintf (stdout,"\n\n Divergence of inner iteation loop is detected. Inner loop is stopped.\n\n");
	  break;
	}
      }
      else
        lsm_quad(lsm_a, lsm_r, lsm_l, double(j+1), norf, Mp->zero,0);*/
    }
    
    if (j==ini || norf>err)
    {
      if (dlambda == dlambdamin)
      {
        if (Mespr==1)  fprintf (stdout,"\n increment cannot be decreased dlambda=%e    norf=%e",dlambda,norf);
        break;
      }
      dlambda/=2.0;
      if (dlambda<dlambdamin)  dlambda=dlambdamin;
      if (Mespr==1)  fprintf (stdout,"\n increment must be modified (dlambda decrease) dlambda=%e    norf=%e",dlambda,norf);
      
      for (j=0;j<n;j++)
      	r[j]=rb[j];

      lambda=blambda;     
    }
    else
    {
      lambda+=dlambda;
      Mm->updateipval ();
      compute_req_val (lcid);
      print_step(lcid, i+1, lambda, f);
      print_flush();
    }
  }
  
  delete [] dr;  delete [] fi;  delete [] fb;  delete [] f;
  print_close();
}



/**  
  Function solves system of nonlinear algebraic
  equations by Newton-Raphson method solved system does not contain time variable.
  Displacement control is assumed and required value of lambda is reached only.
  (initial lambda value is assumed for global Mp->lambda and rlambda is  ).
  
  @param lcid      - required load case id
  @param inilambda - initial value of lambda for displacements
  @param rlambda   - required value of lambda
  @param errl      - required error for reaching rlambda
  @incrdispl       - indicator for increasing Mp->lambda(1=yes/0=no)

  @return The function returns reached load coefficient lambda.

  Created by Tomas Koudelka, 9.11.2005
*/
double displ_controlrv (long lcid, double inilambda, double dmplambda, double rlambda, double errl, long incrdispl)
{
  long i,j,k,n,ni,ini;
  double lambda,blambda,dlambda,dlambdamin,zero,err,norf,norfa;
  double *r,*rb,*dr,*f,*fi,*fb,*fc,*fp, *fir2;


  //  number of rows of the matrix
  n = Ndofm;
  //  maximum number of increments
  ni = Mp->nlman.ninr;
  //  maximum number of iterations in one increment
  ini = Mp->nlman.niilnr;
  //  computer zero
  zero = Mp->zero;
  //  required error in inner loop
  err = Mp->nlman.errnr;
  //  increment size
  dlambda=Mp->nlman.incrnr;
  //  minimum increment size
  dlambdamin=Mp->nlman.minincrnr;

  rb = new double [n];
  f  = new double [n];
  fb = new double [n];
  fi = new double [n];
  fir2 = new double[n];
  dr = new double [n];
  memset (rb,0,n*sizeof(double));
  memset (f,0,n*sizeof(double));
  memset (fi,0,n*sizeof(double));
  memset (dr,0,n*sizeof(double));
  
  //  initialization phase
  r  = Lsrs->give_lhs (lcid);
  fp = Lsrs->give_rhs (lcid*2);
  fc = Lsrs->give_rhs (lcid*2+1);
  lambda=0.0;
  Mp->lambda = inilambda;
  
  // ***************************
  //  main iteration loop  ****
  // ***************************
  print_init(-1, "wt");
  print_step(lcid, 0, 0.0, f);
  print_flush();
  for (i=0;i<ni;i++){
    
    // checking required value of lambda
    if (fabs(lambda-rlambda) < errl)
      break;
    if ((lambda + dlambda) > rlambda)
      dlambda = rlambda - lambda;

    //  backup of left hand side vector
    for (j=0;j<n;j++){
      rb[j]=r[j];
    }
    //  backup of reached lambda parameter
    blambda=lambda;
    if (incrdispl)
      Mp->lambda = inilambda+dmplambda*lambda;
   
    fprintf (stdout,"\n increment %ld   lambda %e,    dlambda %e",i,lambda,dlambda);
    
    //  vector of maximum load and vector of load increment
    if (incrdispl)
      Mp->lambda += dmplambda*dlambda;
    internal_forces(lcid, fir2);

    for (j=0;j<n;j++){
      f[j]=fc[j]+lambda*fp[j];
      fb[j]= f[j] + dlambda*fp[j] - fir2[j];
    }
    
    //  assembling of tangent stiffness matrix
    if ((Mp->nlman.stmat==tangent_stiff) || (i == 0))
      stiffness_matrix (lcid);
    
    //  solution of K(r).v=F
    //Smat->solve_system (Gtm,dr,fb);
    Mp->ssle->solve_system (Gtm,Smat,dr,fb,Out);

    for (j=0;j<n;j++){
      r[j]+=dr[j];
    }
    
    //  computation of internal forces
    internal_forces (lcid,fi);
    
    //  vector of unbalanced forces
    for (j=0;j<n;j++){
      fb[j]=f[j]+dlambda*fp[j];
    }
    norfa=sizev(fb,n);
    for (j=0;j<n;j++){
      fb[j] -= fi[j];
    }
    //  norm of vector of unbalanced forces
    norf=sizev(fb,n);
    if (norfa != 0.0)
      norf /= norfa;
    
    if (norf<err){
      lambda+=dlambda;
      Mm->updateipval ();
      compute_req_val (lcid);
      print_step(lcid, i+1, lambda, f);
      print_flush();
      continue;
    }
    
    //  internal iteration loop
    for (j=0;j<ini;j++){
      
      //  solution of K(r).v=F
      //Smat->solve_system (Gtm,dr,fb);
      Mp->ssle->solve_system (Gtm,Smat,dr,fb,Out);

      for (k=0;k<n;k++){
	r[k]+=dr[k];
      }
      
      //  computation of internal forces
      internal_forces (lcid,fi);
      
      //  vector of unbalanced forces
      for (k=0;k<n;k++){
	fb[k]=f[k]+dlambda*fp[k]-fi[k];
      }
      
      //  norm of vector of unbalanced forces
      norf=sizev(fb,n);
      if (norfa != 0.0)
        norf /= norfa;
      
      fprintf (stdout,"\n increment %ld   inner loop j %ld     norf=%e  dlambda %e",i,j,norf,dlambda);
      
      if (norf<err)  break;
    }
    
    if (j==ini || norf>err)
    {
      if (dlambda == dlambdamin)
      {
        if (Mespr==1)  fprintf (stdout,"\n increment cannot be decreased dlambda=%e    norf=%e",dlambda,norf);
        break;
      }
      dlambda/=2.0;
      if (dlambda<dlambdamin)  dlambda=dlambdamin;
      if (Mespr==1)  fprintf (stdout,"\n increment must be modified (dlambda decrease) dlambda=%e    norf=%e",dlambda,norf);
      
      for (j=0;j<n;j++)
      	r[j]=rb[j];

      lambda=blambda;     
    }
    else
    {
      lambda+=dlambda;
      Mm->updateipval ();
      compute_req_val (lcid);
      print_step(lcid, i+1, lambda, f);
      print_flush();
    }
  }
  print_close();
  
  delete [] dr;  delete [] fi;  delete [] fb;  delete [] f;
  return (lambda);
}



/**
  Function solves system of nonlinear algebraic equations
  by the arc-length method (continuation method).
  Only one right hand side vector is supported with respect
  to nonlinearity and absence of superposition.

  Notation of internal variables:
  - d stands for delta
  - dd stands for capital delta
   
  - fc - nonproportional %vector
  - fp - proportional %vector
  - n - order of the system = the number of unknowns
  
  @param lcid - load case id
  
  original function was created by JK, 25.7.2001
  and modified by Tomas Koudelka in 2009
  
  this is completely rewritten function by JK, 30.8.2010
*/
long arclength (long lcid)
{
  long i,j,k,n,ni,ini,stop,modif,modif_psi,li;
  double dl,dlmax,dlmin,psi,lambda,blambda,dlambda,ddlambda,ddlambda0,psi0;
  double norf,norfp,norv,norfa,zero,ierr;
  double tmp;
  double *r,*ra,*ddr,*u,*v,*f,*fa,*fp,*fc,*fi;
  
  
  //  the number of rows of the matrix = the number of unknowns
  n = Ndofm;
  //  the maximum number of increments in outer iteration loop
  ni = Mp->nlman.nial;
  //  the maximum number of iterations in one increment
  //  the maximum number of iterations in inner loop
  ini = Mp->nlman.niilal;
  //  computer zero
  zero = Mp->zero;
  //  required residual in the inner loop
  ierr = Mp->nlman.erral;
  //  length of the arc
  dl = Mp->nlman.dlal;
  //  the maximum length of the arc
  dlmax = Mp->nlman.dlmaxal;
  //  the minimum length of the arc
  dlmin = Mp->nlman.dlminal;
  //  displacement-loading driving switch
  psi = Mp->nlman.psial;
  

  // *********************************
  //  allocation of auxiliary arrays
  // *********************************
  //  it serves for backup of displacements
  r = new double [n];
  //  ddr = u + ddlambda.v
  ddr = new double [n];
  u = new double [n];
  v = new double [n];
  f = new double [n];
  //  vector of attained load
  fa = new double [n];
  //  vector if internal forces
  fi = new double [n];

  
  //  attained displacements
  ra = Lsrs->give_lhs (lcid);
  //  vector of proportional load
  fp = Lsrs->give_rhs (lcid*2);
  //  vector of constant load
  fc = Lsrs->give_rhs (lcid*2+1);
  
  //  Delta lambda - cumulative increment in one load increment
  //  at the beginning of computation is always zero
  dlambda=0.0;
  
  if (Mp->nlman.hdbackupal == 2){
    //  the arc-length method starts from previously attained state
    //  li - attained number of steps
    //  n - number of rows of the matrix = number of unknowns
    //  lambda - attained coefficient of proportionality
    //  ra - attained displacements
    //  fp - vector of proportional load
    //  
    //  function also updates number of selected DOFs, selected DOFs
    //  function updates strains, stresses and eqother arrays on integration points
    arclopen (li,n,lambda,dl,ra,fp);
  }
  else{
    //  the arclength method starts from the initial state
    //  proportional parameter
    lambda=0.0;
    //  lower index = number of the first step
    li=0;
  }
  
  //  norm of the vector of proportional load
  norfp = loadincr(fp,n);
  modif=0;
  psi0 = psi; // ???

  if (psi<0.0){
    //  parameter psi will be modified during the computation
    modif_psi=1;
  }else{
    //  parameter psi will be constant during the computation
    modif_psi=0;
  }

  // assembling reached load vector
  for (j=0;j<n;j++){
    fa[j]=fc[j]+(lambda+dlambda)*fp[j];
  }
  
  //  printing of state variables
  if (li)
    //  the first step
    print_init(-1, "at");
  else{
    print_init(-1, "wt");
    print_step(lcid, li, lambda, f);
  }
  

  // ***************************
  //  main iteration loop  ****
  // ***************************
  for (i=li;i<ni;i++){

    //  copy of actual step to problem description
    //  it is used in connection with printing of output values
    Mp->istep = i;
    //  backup of left hand side vector
    copyv (ra, r, n);
    //  backup of reached lambda parameter
    blambda=lambda;
    
    
    if (Mespr==1){
      fprintf (stdout,"\n increment %ld,   lambda=%e   dl=%e  psi=%e",i,lambda,dl,psi);
      //fprintf (Out,"\n increment %ld,   lambda=%e   dl=%e",i,lambda,dl);
    }
    
    //  assembling of stiffness matrix
    assemble_stiffness_matrix (lcid,i,-1,li);
    
    //  backup of the fp, in ldl_sky the right hand side will be destroyed
    copyv(fp, f, n);

    //  solution of K(r).v=f_p
    Mp->ssle->solve_system (Gtm,Smat,v,f,Out);

    //  generalized norm of displacement increments
    norv = displincr (v,n);

    if ((i == 0) && (psi < 0.0) && (norfp > zero)){
      //  if psi<0.0 in the input file, the psi is determined in such a way
      //  that both contributions, forces and displacements, are the same
      //  if psi >= 0.0 in the input file, the value from file is used
      psi0 = psi = norv/norfp;
    }
    
    //  compute new dlambda increment
    tmp = norv*norv+psi*psi*norfp*norfp;
    dlambda = dl/sqrt(tmp);
    fprintf(Out, "Step %ld: ddlambda = %e, norv = %e, norfp = %e\n", i, ddlambda, norv, norfp);

    for (j=0;j<n;j++){
      ddr[j]=dlambda*v[j];
      ra[j]+=ddr[j];
      fa[j]=fc[j]+(lambda+dlambda)*fp[j];
    }
    ddlambda=dlambda;
    
    //  computation of internal forces
    internal_forces (lcid,fi);
    //  computation of residuals
    subv(fa, fi, f, n);    
    //  norm of the vector of residuals
    norf=sizev(f,n);
    //  norm of the vector of attained forces
    norfa = sizev(fa, n);
    if (fabs(norfa) > zero){
      //  if the attained forces are nonzero, the norm of residuals is scaled by the norm of forces
      //  this scaling is reasonable because nondimensional quantity is obtained
      norf /= norfa;
    }
    
    if (norf<ierr){
      // ******************************************
      //  no inner iteration loop is required  ***
      // ******************************************
      if (Mespr==1) 	  
        fprintf (stdout,"\n increment %ld   norf=%e  ierr=%e  ddlambda=%e  dlambda=%e",i,norf,ierr, ddlambda, dlambda);
      
      modif++;
      
      if (modif>1){
	//  equilibrium state was established without inner iteration two times
	//  the length of arc can be increased
	dl*=2.0;
	if (dl>dlmax){
          dl=dlmax;
	  if (Mespr==1)
	    fprintf (stdout,"\n length of arc cannot be increased dl=dlmax=%e",dl);
	}
	else{
	  if (Mespr==1)
	    fprintf (stdout,"\n length of arc was increased to dl=%e",dl);
	}
	modif=0;
      }
      
      //  new parameter of proportionality
      lambda+=dlambda;
      
      
      if (modif_psi){
	//  this part modifies parameter psi
	
        // at this point, ddlambda = dlambda
        // store initial values of load coefficient increment
        if (i==0)
          ddlambda0 = ddlambda;

        // load coefficient increment changed sign -> peak was attained ->
        // actualize values of stored initail load coefficient increment and initial proportional coefficient
        if ((sgn(ddlambda0) != sgn(ddlambda)) && (sgn(ddlambda) != 0.0)){
	  psi0= psi;
          ddlambda0 = ddlambda;
        }
	
        // automatic correction of proportional coefficient
        // with respect to actual and initial load coefficient increment
        if (ddlambda0 != 0.0)
          psi = psi0*fabs(ddlambda/ddlambda0);
      }


      //  update of variables defined in integration points
      Mm->updateipval ();
      compute_req_val (lcid); 
      
      //  printing of state variables
      print_step(lcid, i+1, lambda, f);
      print_flush();
    }
    else{
      // ****************************
      //  inner iteration loop  ****
      // ****************************
      if (Mespr==1) 	  
        fprintf (stdout,"\n inner loop: 1 norf=%e  ierr=%e  ddlambda=%e  dlambda=%e",norf,ierr,ddlambda, dlambda);

      stop=0;

      for (j=0;j<ini;j++){
	//  copy of actual step to problem description
	//  it is used in connection with printing of output values
        Mp->jstep = j;
	
	//  eventual update of stiffness matrix
	assemble_stiffness_matrix (lcid,i,j,li);
	
	//  back substitution
	Mp->ssle->solve_system (Gtm,Smat,u,f,Out);
	
	//  backup of ddr to array f
        copyv(ddr, f, n);
	//  ddr = ddr+u
        addv(ddr, u, n);
	
	//  determination of increment of load parameter
	determine_dlambda (ddr,v,f,n,ddlambda,psi,norfp,dl,dlambda,stop);
	
	
	for (k=0;k<n;k++){
	  ddr[k]+=dlambda*v[k];
	  ra[k]+=u[k]+dlambda*v[k];
	  fa[k]+=dlambda*fp[k];
	}
	ddlambda+=dlambda;
	
	//  computation of internal forces
	internal_forces (lcid,fi);
	//  computation of residual forces
        subv(fa, fi, f, n);
	//  norm of the vector of residuals
	norf=sizev(f,n);
	//  norm of the vector of attained forces
	norfa = sizev(fa, n);
        if (fabs(norfa) > zero){
	  //  if the attained forces are nonzero, the norm of residuals is scaled by the norm of forces
	  //  this scaling is reasonable because nondimensional quantity is obtained
          norf /= norfa;
	}
	
	if (Mespr==1)
	  fprintf (stdout,"\n         %5ld norf=%e  ierr=%e  ddlambda=%e dlambda=%e",j+1,norf,ierr,ddlambda, dlambda);
	
	if (norf<ierr){
	  // **************************
	  //  equilibrium is attained
	  // **************************
	  
	  //  new load parameter
	  lambda+=ddlambda;
	  
	  if (modif_psi){
	    //  this part modifies parameter psi
	    
	    // at this point, ddlambda = dlambda
	    // store initial values of load coefficient increment
	    if (i==0)
	      ddlambda0 = ddlambda;
	    
	    // load coefficient increment changed sign -> peak was attained ->
	    // actualize values of stored initail load coefficient increment and initial proportional coefficient
	    if ((sgn(ddlambda0) != sgn(ddlambda)) && (sgn(ddlambda) != 0.0)){
	      psi0= psi;
	      ddlambda0 = ddlambda;
	    }
	    
	    // automatic correction of proportional coefficient
	    // with respect to actual and initial load coefficient increment
	    if (ddlambda0 != 0.0)
	      psi = psi0*fabs(ddlambda/ddlambda0);
	  }
	  
	  //  equilibrium was reached correctly, the length of arc need not be decreased
	  stop=1; 
	  
	  //  update of variables defined in integration points
	  Mm->updateipval ();
	  compute_req_val (lcid);
	  
	  //  printing of state variables
          print_step(lcid, i+1, lambda, f);
          print_flush();
	  break;
	}
      }
      // ********************
      //  end of inner loop
      // ********************
      
      //  there is no modification of the length of arc because at least one iteration was performed
      modif=0;
      if (stop==0){
	//  equilibirum was not reached in inner loop,
	//  the arc length has to be decreased

	if (dl==dlmin){
	  if (Mespr==1){
	    fprintf (stdout,"\n arc length cannot be decreased dl=dlmin=%e",dl);
	  }
	  break;
	}

	dl/=2.0;
	if (dl<dlmin){
          dl=dlmin;  
	  if (Mespr==1){
	    fprintf (stdout,"\n arc length was decreased dl=dlmin=%e",dl);
	  }
        }else{
	  if (Mespr==1){
	    fprintf (stdout,"\n arc length was decreased dl=%e",dl);
	  }
	}
	//  restoring of left hand side vector
        copyv(r, ra, n);
	//  restoring of lambda parameter
	lambda=blambda;
      }
    }

  }
  // ********************
  //  end of outer loop
  // ********************
  
  
  if (Mp->nlman.hdbackupal==1){
    //  backup of the attained state is required
    arclsave (i,n,lambda,dl,ra,fp);
  }
  
  if ((i < ni-1) && (norf >= ierr)){    
    if (Mespr==1)  fprintf (stderr,"\n\n load increment is less than minimum load increment");
    if (Mespr==1)  fprintf (stderr,"\n computation failed (file %s, line %d)\n",__FILE__,__LINE__);
    if (Mespr==1)  fprintf (stderr,"\n FORCED output of results from this step is performed\n");
    compute_req_val (lcid);
    print_step_forced(lcid, i+1, lambda, f);
    print_flush();
  }
  
  print_close();
  
  //  cleaning of all arrays
  delete [] r;		    
  delete [] fi;  
  delete [] fa;  
  delete [] f;
  delete [] v;  
  delete [] u;  
  delete [] ddr;
  
  return (i);
}

/**
  function assembles the stiffness %matrix
  it depends on the variable Mp->nlman.stmat
  for possible values see alias.h
  
  @param lcid - load case id
  @param i - index in outer loop
  @param j - index in inner loop
  @param li - lower index of the outer loop = number of the first outer increment
            li=0 if the computation starts from the initial state
	    li>0 if the computation starts from backup

  JK, TKo, 30.8.2010	    
*/
void assemble_stiffness_matrix (long lcid,long i,long j,long li)
{
  switch (Mp->nlman.stmat){
  case initial_stiff:{
    if (i==li && j<0){
      //  this condition describes the first step in the computation
      stiffness_matrix (lcid);
    }
    break;
  }
  case tangent_stiff:{
    stiffness_matrix (lcid);
    break;
  }
  case secant_stiff:{
    stiffness_matrix (lcid);
    break;
  }
  case incr_tangent_stiff:{
    if (j<0){
      //  the stiffness matrix is updated only after outer step
      //  this fact is indicated by the variable j which is set to negative number
      stiffness_matrix (lcid);
    }
    break;
  }
  case ijth_tangent_stiff:{
    if (j<0){
      //  this is modification in outer loop
      if ((i%Mp->nlman.ithstiffchange == 0) || (i==li))
	stiffness_matrix (lcid);
    }
    else{
      //  this is modification in inner loop
      if (j%Mp->nlman.jthstiffchange == 0)
	stiffness_matrix (lcid);
    }
    break;
  }
  default:{
    print_err("unknown type of stiffness matrix is required",__FILE__,__LINE__,__func__);
  }
  }
  
}

/**
   function determines increment of lambda (delta lambda)
   
   @param ddr - Delta r + u
   @param v - 
   @param ddrprev - Delta r (not updated by the vector u)
   @param n - number of unknowns
   @param ddlambda - Delta lambda
   @param psi - 
   @param norfp - the norm of proportional vector
   @param dl - length of arc
   @param dlambda - delta lambda (output of this function)
   @param stop - indicator of end of iteration (output of this function)
   
   JK, 30.8.2010
*/
void determine_dlambda (double *ddr,double *v,double *ddrprev,long n,double ddlambda,double psi,double norfp,double dl,double &dlambda,long &stop)
{
  long k,numr;
  double l1,l2,a0,a1,a2,ss1,ss2,ss3,ss4,ss5,nom,denom;
  
  switch (Mp->nlman.dlam){
  case nodetermination:{
    print_err("some reasonable type of lambda determination has to be selected",__FILE__,__LINE__,__func__);
    break;
  }
  case minvalue:
  case maxvalue:
  case minangle:{
    //  coefficient of quadratic equation
    quadeqcoeff (ddr,v,n,ddlambda,psi,norfp,dl,a0,a1,a2);
    
    //  solution of quadratic equation
    numr = solv_polynom_2(a2, a1, a0, l1, l2);
    switch (numr){
    case -1:{
      print_err("infinite number of solution of constrained condition", __FILE__, __LINE__, __func__);
      print_err("(all coefficients of quadratic equation are equal to zero", __FILE__, __LINE__, __func__);
      abort();
      break;
    }
    case 0:{
      print_err("nonsolvable constrained condition in function arclength", __FILE__, __LINE__, __func__);
      stop=0;
      break;
    }
    case 1:{
      dlambda = l1;
      break;
    }
    default:{}
    }
    
    break;
  }
  case linearizedmeth:{
    break;
  }
  default:{
    print_err("unknown type of detemination of lambda parameter is required",__FILE__,__LINE__,__func__);
  }
  }
  
  
  switch (Mp->nlman.dlam){
  case nodetermination:{
    print_err("some reasonable type of lambda determination has to be selected",__FILE__,__LINE__,__func__);
    break;
  }
  case minvalue:{
    if (fabs(l1)>fabs(l2))  dlambda=l2;
    else dlambda=l1;
    break;
  }
  case maxvalue:{
    if (fabs(l1)>fabs(l2))  dlambda=l1;
    else dlambda=l2;
    break;
  }
  case minangle:{
    ss1=0.0;  ss2=0.0;
    ss3=0.0;  ss4=0.0;  ss5=0.0;
    for (k=0;k<n;k++){
      ss1+=(ddr[k]+l1*v[k])*ddrprev[k];
      ss2+=(ddr[k]+l2*v[k])*ddrprev[k];
      ss3+=(ddr[k]+l1*v[k])*(ddr[k]+l1*v[k]);
      ss4+=(ddr[k]+l2*v[k])*(ddr[k]+l2*v[k]);
      ss5+=ddrprev[k]*ddrprev[k];
    }
    if (ss1/sqrt(ss3)/sqrt(ss5)>ss2/sqrt(ss4)/sqrt(ss5))  
      dlambda=l1;
    else                          
      dlambda=l2;
    
    break;
  }
  case linearizedmeth:{
    nom=0.0;
    denom=0.0;
    for (k=0;k<n;k++){
      nom-=ddrprev[k]*(ddr[k]-ddrprev[k]);
      denom+=ddrprev[k]*v[k];
    }
    denom+=psi*psi*norfp*norfp*ddlambda;
    
    dlambda=nom/denom;
    break;
  }
  default:{
    print_err("unknown type of detemination of lambda parameter is required",__FILE__,__LINE__,__func__);
  }
  }
  
}

/**
  Function solves system of nonlinear algebraic equation
  by arc-length method (continuation method).
  Only one right hand side vector is supported with respect
  to nonlinearity and absence of superposition.

  Notation of internal variables:
  - d stands for delta
  - dd stands for capital delta
   
  - fc - nonproportional %vector
  - fp - proportional %vector
  - n - order of the system
   
  @param lcid - load case id
   
  Created by JK, 25.7.2001
  Modified by Tomas Koudelka,
*/
long arclength_old (long lcid,long adaptcontrol)
{
  long i,j,k,n,ni,ini,stop,stopj,modif,li,newmesh, numr;
  double a0,a1,a2,l1,l2,dl,dlmax,dlmin,psi,lambda,blambda,dlambda,ddlambda,ddlambda0,psi0;
  double norf,norfp,norv,norfa,zero,ierr;
  double lambdao,ss1,ss2,ss3,ss4,ss5,tmp;
  double *r,*ra,*ddr,*u,*v,*f,*fa,*fp,*fc,*fi;
  
  
  
  //  number of rows of the matrix
  n = Ndofm;
  //  maximum number of increments
  ni = Mp->nlman.nial;
  //  maximum number of iterations in one increment
  ini = Mp->nlman.niilal;
  //  computer zero
  Mp->zero = zero = 1e-6;
  //  required error in inner loop
  ierr = Mp->nlman.erral;
  //  length of arc
  dl = Mp->nlman.dlal;
  //  maximum length of arc
  dlmax = Mp->nlman.dlmaxal;
  //  minimum length of arc
  dlmin = Mp->nlman.dlminal;
  //  displacement-loading driving switch
  psi = Mp->nlman.psial;
  
  dlambda=0.0;
  
  //  allocation of auxiliary arrays
  r = new double [n];
  ddr = new double [n];
  u = new double [n];
  v = new double [n];
  f = new double [n];
  fa = new double [n];
  fi = new double [n];
  
  //  pokus
  //double *zaloha,*zaloha2;
  //zaloha = new double [n];
  //zaloha2 = new double [n];
  //for (i=0;i<n;i++){
  //zaloha[i]=0.0;
  //zaloha2[i]=0.0;
  //}
  // konec pokusu
  
  //  initialization phase
  ra = Lsrs->give_lhs (lcid);
  fp = Lsrs->give_rhs (lcid*2);
  fc = Lsrs->give_rhs (lcid*2+1);
  
  if (Mp->nlman.hdbackupal == 2){            // termit
    arclopen (li,n,lambda,dl,ra,fp);
  }
  else if (adaptcontrol >= 2){
    arclopen_adapt (li,lambda,blambda,dl);
  }
  else{
    lambda=0.0;
    lambdao=0.0;
    li=0;
  }
  
  //refresh_plast ();  // termit
  
  //  Mp->ns;

  //  norm of proportionality vector
  norfp = loadincr(fp,n);
  //  lenght of arc modification variable
  modif=0;
  psi0 = psi;

  // assembling reached load vector
  for (j=0;j<n;j++){
    fa[j]=fc[j]+(lambda+dlambda)*fp[j];
  }
  if (li)
    print_init(-1, "at");
  else
  {
    print_init(-1, "wt");
    print_step(lcid, li, lambda, f);
  }
  
  // ***************************
  //  main iteration loop  ****
  // ***************************
  for (i=li;i<ni;i++){

    Mp->istep = i;
    stop=1; // termit
    //  backup of left hand side vector
    copyv (ra, r, n);
    //  backup of reached lambda parameter
    blambda=lambda;
    
    
    if (Mespr==1){
      fprintf (stdout,"\n increment %ld,   lambda=%e   dl=%e  psi=%e",i,lambda,dl,psi);
      //fprintf (Out,"\n increment %ld,   lambda=%e   dl=%e",i,lambda,dl);
    }
    
    //  assembling of tangent stiffness matrix
    if ((Mp->nlman.stmat==tangent_stiff) || (i == li)){
      stiffness_matrix (lcid);
    }
    
    //  backup of the fp, in ldl_sky the right hand side will be destroyed
    copyv(fp, f, n);
    
    //  solution of K(r).v=F
    Mp->ssle->solve_system (Gtm,Smat,v,f,Out);
    
    //  generalized norm of displacement increments
    norv = displincr (v,n);
    
    if ((i == 0) && (psi < 0.0) && (norfp != Mp->zero))
      psi0 = psi = sqrt(norv*norv/norfp/norfp);
 
    //  compute new dlambda increment
    tmp = norv*norv+psi*psi*norfp*norfp;
    dlambda = dl/sqrt(tmp);
    fprintf(Out, "Step %ld: ddlambda = %e, norv = %e, norfp = %e\n", i, ddlambda, norv, norfp);
    
    for (j=0;j<n;j++){
      ddr[j]=dlambda*v[j];
      ra[j]+=ddr[j];
      fa[j]=fc[j]+(lambda+dlambda)*fp[j];
    }
    ddlambda=dlambda;
    
    //  computation of internal forces
    internal_forces (lcid,fi);
    //  computation of residual forces
    subv(fa, fi, f, n);
    //  norm of the residual forces
    norf=sizev(f,n);
    //  norm of the attained forces
    norfa = sizev(fa, n);
    if (fabs(norfa) > zero){
      //  if the attained forces are nonzero, the norm of residuals is scaled by the norm of forces
      //  this scaling is reasonable because nondimensional quantity is obtained
      norf /= norfa;
    }
    
    if (norf<ierr){
      // ******************************************
      //  no inner iteration loop is required  ***
      // ******************************************
      if (Mespr==1) 	  
        fprintf (stdout,"\n increment %ld   norf=%e  ierr=%e  ddlambda=%e  dlambda=%e",i,norf,ierr, ddlambda, dlambda);
      
      modif++;
      
      if (modif>1){
	//  equilibrium state was established without inner iteration two times
	//  the length of arc can be increased
	dl*=2.0;
	if (dl>dlmax){
          dl=dlmax;
	  if (Mespr==1)
	    fprintf (stdout,"\n length of arc cannot be increased dl=dlmax=%e",dl);
	}
	else{
	  if (Mespr==1)
	    fprintf (stdout,"\n length of arc was increased to dl=%e",dl);
	}
	modif=0;
      }
      
      //  new parameter of proportionality
      lambda+=dlambda;
      if (i==0)
        ddlambda0 = dlambda;
      if ((sgn(ddlambda0) != sgn(dlambda)) && (sgn(dlambda) != 0.0))
      {
        psi0= psi;
        ddlambda0 = dlambda;
      }
      if (ddlambda0 != 0.0)
        psi = psi0*fabs(dlambda/ddlambda0);
      
      //  update of variables defined in integration points
      Mm->updateipval ();
      compute_req_val (lcid);
      
      //  printing of state variables
      print_step(lcid, i+1, lambda, f);
      print_flush();
    }
    else{
      // ****************************
      //   inner iteration loop  ****
      // ****************************
      if (Mespr==1) 	  
        fprintf (stdout,"\n inner loop: 1 norf=%e  ierr=%e  ddlambda=%e  dlambda=%e",norf,ierr,ddlambda, dlambda);

      stop=0;
      stopj=0;

      for (j=0;j<ini;j++){
        Mp->jstep = j;
        Mespr = 0;
	if ((Mp->nlman.stmat==tangent_stiff) && (j%10 == 0) && (j != 0))
	  stiffness_matrix (lcid);
        Mespr = 1;

	//  back substitution
	//  f contains residual forces
	Mp->ssle->solve_system (Gtm,Smat,u,f,Out);
	
        copyv(ddr, f, n);
	//  update of displacement increments
        addv(ddr, u, n);
	//  coefficient of quadratic equation
	quadeqcoeff (ddr,v,n,ddlambda,psi,norfp,dl,a0,a1,a2);
	
        fprintf (Out," \t j = %ld: (ddr,v)   %15.10e",j, ss(ddr,v,n));
	fprintf (Out," (ddr,ddr) %15.10e",ss(ddr,ddr,n));
	fprintf (Out," ddlambda  %15.10e\n",ddlambda);
	
	//  solution of quadratic equation
        numr = solv_polynom_2(a2, a1, a0, l1, l2);
        switch (numr)
	{
	  case -1:
            print_err("infinite number of solution of constrained condition", __FILE__, __LINE__, __func__);
            abort();
            break;
	  case 0:
            print_err("nonsolvable constrained condition in function arclength", __FILE__, __LINE__, __func__);
            stop=0;
            stopj = 1;
            break;
    	  case 1:
            dlambda = l1;
            break;
	  default:
            ss1=0.0;  ss2=0.0;
	    ss3=0.0;  ss4=0.0;  ss5=0.0;
	    for (k=0;k<n;k++){
	      ss1+=(ddr[k]+l1*v[k])*f[k];
	      ss2+=(ddr[k]+l2*v[k])*f[k];
	      ss3+=(ddr[k]+l1*v[k])*(ddr[k]+l1*v[k]);
	      ss4+=(ddr[k]+l2*v[k])*(ddr[k]+l2*v[k]);
	      ss5+=f[k]*f[k];
	    }
	    if (ss1/sqrt(ss3)/sqrt(ss5)>ss2/sqrt(ss4)/sqrt(ss5))  
	      dlambda=l1;
	    else                          
	      dlambda=l2;
	}
        if (stopj)
          break;
	//if (fabs(l1)>fabs(l2))  dlambda=l2;
	//else dlambda=l1;
	//if (fabs(l1)>fabs(l2))  dlambda=l1;
	//else dlambda=l2;

	for (k=0;k<n;k++){
	  ddr[k]+=dlambda*v[k];
	  ra[k]+=u[k]+dlambda*v[k];
	  fa[k]+=dlambda*fp[k];
	}
	ddlambda+=dlambda;
	
	//  computation of internal forces
	internal_forces (lcid,fi);
	
        subv(fa, fi, f, n);	
	norf=sizev(f,n);

	//  patek 27.8.2010
	if (i==18){
	  compute_req_val (lcid);
	  print_step_forced(lcid, j+50, lambda, f);
	  print_flush();
	}
	//  patek 27.8.2010

	norfa = sizev(fa, n);
        if (fabs(norfa) > Mp->zero)
          norf /= norfa;
	
	if (Mespr==1)
	  fprintf (stdout,"\n         %5ld norf=%e  ierr=%e  ddlambda=%e dlambda=%e",j+1,norf,ierr,ddlambda, dlambda);

	if (norf<ierr){
	  lambda+=ddlambda;
          if (i==0)
            ddlambda0 = ddlambda;
          if ((sgn(ddlambda0) != sgn(ddlambda)) && (sgn(ddlambda) != 0.0))
    	  {
            ddlambda0 = ddlambda;
            psi0 = psi;
	  }
          if (ddlambda0 != 0.0)
            psi = psi0*fabs(ddlambda/ddlambda0);
	  Mm->updateipval ();
	  stop=1; 
	  compute_req_val (lcid);
          print_step(lcid, i+1, lambda, f);
          print_flush();
	  break;
	}
      }
      modif=0;
      if (stop==0){
	//  modification of the arc length
	dl/=2.0;
	if (dl<dlmin){
          dl=dlmin;  
          break; 
        }
	if (Mespr==1){
	  fprintf (stdout,"\n arc length must be modified (dl decrease) dl=%e",dl);
	  //fprintf (Out,"\n arc length must be modified (dl decrease) dl=%e",dl);
	}
	//  restoring of left hand side vector
        copyv(r, ra, n);
	//  restoring of lambda parameter
	lambda=blambda;
      }
    }
    

    if (stop==0)
      continue;
    
    if (Mp->adaptflag & 16 && i%10 && Mm->plast)
      continue;
    
    newmesh = 0;
    if (Mp->adaptivity && Mp->nlman.nial > (i+1) && !( (Mp->adaptflag & 16) && (Mp->nlman.nial==100 || Mp->nlman.nial==1))){
      
      // print of fxlambda second part is in adaptivity.cpp
      //FILE *ff;
      //sprintf (file,"%sfxlambda.dat",Mp->path);
      //ff = fopen (file,"a");
      //if (ff==NULL) fprintf (stderr,"\n ff file has not been opened.\n\n");
      //fprintf (ff,"  %20.10f",lambda);
      //fclose (ff);
      
      long width = 1 + (long)log10((double)Mp->nlman.nial);
      newmesh = Ada->run (1,width,i);
    }
    
    if (newmesh){
      i++;             // toto se da odstranit
      break;
    }
    // timret
    //print_output(lcid);
  }
  
  // ------------------------------------
  //  finish of main iteration loop  ----
  // ------------------------------------
  
  //fprintf (Out,"\n\n blb  %lf",Lsrs->lhs[0]);

  
  if (Mp->nlman.hdbackupal==1)
    arclsave (i,n,lambda,dl,ra,fp);

  // termit
  if (Mp->adaptivity)
  {
    if ((Mp->adaptflag & 16) && (Mp->nlman.nial==100 || Mp->nlman.nial==1))
      Ada->run (0,0,0);  // only for testing
    else
    {
      if (i < ni)
	arclsave_adapt (i,lambda,0,dl);
    }
  }
  if ((i < ni-1) && (norf >= ierr))
  {    
    if (Mespr==1)  fprintf (stderr,"\n\n load increment is less than minimum load increment");
    if (Mespr==1)  fprintf (stderr,"\n computation failed (file %s, line %d)\n",__FILE__,__LINE__);
    if (Mespr==1)  fprintf (stderr,"\n FORCED output of results from this step is performed\n");
    compute_req_val (lcid);
    print_step_forced(lcid, i+1, lambda, f);
    print_flush();
  }
  //if (Mp->adaptivity)   arclsave_adapt (i,blambda,ddlambda,dl);
  // timret
  print_close();
  delete [] r;		    
  delete [] fi;  
  delete [] fa;  
  delete [] f;
  delete [] v;  
  delete [] u;  
  delete [] ddr;
  
  return (i);
}


/**
  Function computes coefficients of the quadratic equation used in the arclength method.
  The coefficients are computed with respect to type of load/displacement control.

  @param ddr - increment of load coefficient lambda for the given step
  @param v   - %vector of displacement increments
  @param n   - total number of DOFs (i.e. size of ddr and v arrays)
  @param ddlambda - correction of increment of load coefficient lambda
  @param psi   - proportional weight coeffient (swithces between load/displacement control)
  @param norfp - norm of proportional load %vector
  @param dl - length of arc
  @param a0 - zero order coefficient (output)
  @param a1 - first order coefficient (output)
  @param a2 - second order coefficient (output)

  @return The function returns evaluated coefficient in the parameters a0, a1 and a2.
 
  Created by JK,
*/
void quadeqcoeff (double *ddr,double *v,long n,double ddlambda,double psi,double norfp,double dl,
		  double &a0,double &a1,double &a2)
{
  long i;
  double norddr, norv;
  
  switch (Mp->nlman.displnorm){
  case alldofs:{
    a2=ss(v,v,n)+psi*psi*norfp*norfp;
    a1=2.0*ss(ddr,v,n)+2.0*ddlambda*psi*psi*norfp*norfp;
    a0=ss(ddr,ddr,n)+ddlambda*ddlambda*psi*psi*norfp*norfp-dl*dl;
    break;
  }
  case seldofs:
  case seldofscoord:
  case selecnodes:{
    a0=0.0;  a1=0.0;  a2=0.0;
    for (i=0;i<Mp->nlman.nsdofal;i++){
      a0+=ddr[Mp->nlman.seldofal[i]]*ddr[Mp->nlman.seldofal[i]];
      a1+=2.0*ddr[Mp->nlman.seldofal[i]]*v[Mp->nlman.seldofal[i]];
      a2+=v[Mp->nlman.seldofal[i]]*v[Mp->nlman.seldofal[i]];
    }    
    a0 += ddlambda*ddlambda*psi*psi*norfp*norfp - dl*dl;
    a1 += 2.0*ddlambda*psi*psi*norfp*norfp;
    a2 += psi*psi*norfp*norfp;
    break;
  }
  case nodesdistincr:{
    norddr = displincr(ddr, n);
    norv   = displincr(v, n);
    a0 = norddr*norddr + ddlambda*ddlambda*norfp*norfp*psi*psi-dl*dl;
    a1 = 2.0*(norv*norddr + ddlambda*norfp*norfp*psi*psi);
    a2 = norv*norv + norfp*norfp*psi*psi;
    break;
  }
  default:
    print_err("unknown norm of displacement increment is required", __FILE__, __LINE__, __func__);
  }
}



/**
  The function initializes dof numbers for the particular
  types of displacement control. It is used in the arclength method.

  @return The function does not return anything.
  
  Created by JK,
*/
void seldofinit ()
{
  long i,j,k,l,ndofn;
  double x1,x2,y1,y2,z1,z2,length;
  
  switch (Mp->nlman.displnorm){
  case alldofs:{  break; }
  case seldofs:
  case seldofscoord:{
    for (i=0;i<Mp->nlman.nsdofal;i++){
      j=Mp->nlman.seldofal[i];
      Mp->nlman.seldofal[i]=Mt->give_dof (Mp->nlman.selnodal[i],j)-1;
    }
    break;
  }
  case selecnodes:{
    Mp->nlman.nsdofal=0;
    for (i=0;i<Mp->nlman.nsnal;i++){
      Mp->nlman.nsdofal+=Mt->give_ndofn (Mp->nlman.selnodal[i]);
    }
    Mp->nlman.seldofal = new long [Mp->nlman.nsdofal];
    k=0;
    for (i=0;i<Mp->nlman.nsnal;i++){
      ndofn=Mt->give_ndofn (Mp->nlman.selnodal[i]);
      for (j=0;j<ndofn;j++){
	l=Mt->give_dof (Mp->nlman.selnodal[i],j);
	if (l>0){
	  Mp->nlman.seldofal[k]=l-1;
	  k++;
	}
      }
    }
    Mp->nlman.nsdofal=k;
    break;
  }
  case nodesdistincr:{
    Mp->nlman.nsdofal=0;
    for (i=0;i<Mp->nlman.nsnal;i++){
      Mp->nlman.nsdofal+=Mt->give_ndofn (Mp->nlman.selnodal[i]);
    }
    Mp->nlman.seldofal = new long [Mp->nlman.nsdofal];
    k=0;
    for (i=0;i<Mp->nlman.nsnal;i++){
      ndofn=Mt->give_ndofn (Mp->nlman.selnodal[i]);
      for (j=0;j<ndofn;j++){
	Mp->nlman.seldofal[k]=Mt->give_dof (Mp->nlman.selnodal[i],j);
	k++;
      }
    }
    x1=Gtm->gnodes[Mp->nlman.selnodal[0]].x;    x2=Gtm->gnodes[Mp->nlman.selnodal[1]].x;
    y1=Gtm->gnodes[Mp->nlman.selnodal[0]].y;    y2=Gtm->gnodes[Mp->nlman.selnodal[1]].y;
    z1=Gtm->gnodes[Mp->nlman.selnodal[0]].z;    z2=Gtm->gnodes[Mp->nlman.selnodal[1]].z;
    length=sqrt((x2-x1)*(x2-x1)+(y2-y1)*(y2-y1)+(z2-z1)*(z2-z1));
    Mp->nlman.nxal=(x2-x1)/length;  Mp->nlman.nyal=(y2-y1)/length;  Mp->nlman.nzal=(z2-z1)/length;
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown displacement norm is required in function seldofinit (%s, line %d)",__FILE__,__LINE__);
  }
  }
}



/**
  The function computes generalized norm of %vector of 
  displacemnt increments. The function returns either %vector norm computed
  from the selected componets or it returns increment of distance between 
  selected nodes. It is used in the arclength method.

  @param displincr - array with components of %vector of displacement increments
  @param n - number of components of displincr array

  @return The function returns required norm.

  Created by JK,
*/
double displincr (double *displincr,long n)
{
  long i;
  double norm,u1,u2,v1,v2,w1,w2;
  
  switch (Mp->nlman.displnorm){
  case alldofs:{
    norm = ss (displincr,displincr,n);
    norm=sqrt (norm);
    break;
  }
  case seldofs:
  case seldofscoord:
  case selecnodes:{
    norm=0.0;
    for (i=0;i<Mp->nlman.nsdofal;i++){
      norm+=displincr[Mp->nlman.seldofal[i]]*displincr[Mp->nlman.seldofal[i]];
    }
    norm=sqrt (norm);
    break;
  }
  case nodesdistincr:{
    if (Mp->nlman.probdimal==2){
      u1=0.0;  u2=0.0;  v1=0.0;  v2=0.0;
      if (Mp->nlman.seldofal[0]>0)  u1=displincr[Mp->nlman.seldofal[0]-1];
      if (Mp->nlman.seldofal[1]>0)  v1=displincr[Mp->nlman.seldofal[1]-1];
      if (Mp->nlman.seldofal[2]>0)  u2=displincr[Mp->nlman.seldofal[2]-1];
      if (Mp->nlman.seldofal[3]>0)  v2=displincr[Mp->nlman.seldofal[3]-1];
      
      norm=(u2-u1)*Mp->nlman.nxal + (v2-v1)*Mp->nlman.nyal;
    }
    if (Mp->nlman.probdimal==3){
      u1=0.0;  u2=0.0;  v1=0.0;  v2=0.0;  w1=0.0;  w2=0.0;
      if (Mp->nlman.seldofal[0]>0)  u1=displincr[Mp->nlman.seldofal[0]-1];
      if (Mp->nlman.seldofal[1]>0)  v1=displincr[Mp->nlman.seldofal[1]-1];
      if (Mp->nlman.seldofal[2]>0)  w1=displincr[Mp->nlman.seldofal[2]-1];
      if (Mp->nlman.seldofal[3]>0)  u2=displincr[Mp->nlman.seldofal[3]-1];
      if (Mp->nlman.seldofal[4]>0)  v2=displincr[Mp->nlman.seldofal[4]-1];
      if (Mp->nlman.seldofal[5]>0)  w2=displincr[Mp->nlman.seldofal[5]-1];
      
      norm=(u2-u1)*Mp->nlman.nxal + (v2-v1)*Mp->nlman.nyal + (w2-w1)*Mp->nlman.nzal;
    }
    break;
  }
  default:
    print_err("unknown norm of displacement increment is required", __FILE__, __LINE__, __func__);
  }
  return norm;
}



/**
  The function computes generalized norm of load %vector.
  It is used in the arclength method.

  @param loadincr - array with components of load %vector
  @param n - the number of components of displincr array

  @return The function returns required norm.

  Created by JK,
*/
double loadincr (double *loadincr,long n)
{
  long i;
  double norm;

  norm = 0.0;
  switch (Mp->nlman.displnorm)
  {
    case alldofs:
    case nodesdistincr:
      norm = ss (loadincr,loadincr,n);
      break;
    case seldofs:
    case seldofscoord:
    case selecnodes:
      for (i=0;i<Mp->nlman.nsdofal;i++)
        norm+=loadincr[Mp->nlman.seldofal[i]]*loadincr[Mp->nlman.seldofal[i]];
      break;
    default:
      break;
  }
  norm = sqrt(norm);
  return norm;
}



/**  
  Function solves system of nonlinear algebraic equations by 
  Newton-Raphson method. Solved system does not contain time variable.
  The method retarts analysis from the saved data of the time dependent 
  analysis. Internal data rearangement is performed with respect of 
  solved problem in Hinkley nuclear power plant.
  
  @param lcid      - required load case id

  @return The function returns reached load coefficient lambda.

  Created by JK, Tomas Koudelka
*/
void newton_raphson_restart (long lcid)
  //  function solves system of nonlinear algebraic
  //  equations by Newton-Raphson method
  //  solved system does not contain time variable
  //
  //  16.8.2001
{
  long i,j,k,n,ni,ini,nii;
  double lambda,blambda,dlambda,dlambdamin,zero,err,norf,norfa;
  double *r,*rb,*dr,*f,*fi,*fb,*fc,*fp;


  //  number of rows of the matrix
  n = Ndofm;
  //  maximum number of increments
  ni = Mp->nlman.ninr;
  // number of initial increments
  nii = Mp->nlman.nienr;
  //  maximum number of iterations in one increment
  ini = Mp->nlman.niilnr;
  //  computer zero
  zero = Mp->zero;
  //  required error in inner loop
  err = Mp->nlman.errnr;
  //  increment size
  dlambda=Mp->nlman.incrnr;
  //  minimum increment size
  dlambdamin=Mp->nlman.minincrnr;

  rb = new double [n];
  f  = new double [n];
  fb = new double [n];
  fi = new double [n];
  dr = new double [n];
  memset (rb,0,n*sizeof(double));
  memset (f,0,n*sizeof(double));
  memset (fi,0,n*sizeof(double));
  memset (dr,0,n*sizeof(double));
  
  //  initialization phase
  r  = Lsrs->give_lhs (lcid);
  fp = Lsrs->give_rhs (lcid*2);
  fc = Lsrs->give_rhs (lcid*2+1);
  lambda=0.0;




  long ncompstr;
  double tmp;
  FILE *aux;

  // if restart from backup file is required
  if (Mp->nlman.hdbr)
  {
    aux = fopen (Mp->nlman.backupfname, "rt");
    if (aux == NULL)
    {
      fprintf(stderr, "\n\nError - unable to open backup file %s\n", Mp->nlman.backupfname);
      fprintf(stderr, "file %s, line %d\n", __FILE__, __LINE__);
      fprintf(stderr, "Program will be terminated\n");
      abort();
    }
    // searching required backup
    fprintf(stdout, "\n");
    mtsolver_restore(r,fc,i,tmp,tmp,Mp->timecon,n);
    /*
    for (k=0; k<Mp->nlman.hdbid; k++)
    {
      fprintf(stdout, "\r reading backup record %ld", k+1);
      fflush(stdout);
      mtsolver_restore(aux, r, fc, i, tmp, j, 39, 8);
    }
    fprintf(stdout, "\n backup time: %e\n", tmp);
    if (j != Ndofm)
    {
      fprintf(stderr, "\n\nError - incorrect number of DOFs is stored in backup file\n");
      fprintf(stderr, "file %s, line %d\n", __FILE__, __LINE__);
    }
    */
/*
  Begining of Hinkley modification
*/
    for (i=0; i<Mm->tnip; i++)
    {
      if (Mm->ip[i].ncompeqother >= 8)
      {
        ncompstr = Mm->ip[i].ncompstr;
        for (j=0; j<ncompstr; j++)
          Mm->ip[i].stress[j] = Mm->ip[i].eqother[ncompstr+j];
      }
    }
  }

  Mp->strcomp = 0;
  internal_forces(lcid, fc);
  for (i=0; i < Ndofm; i++)
    fc[i] *= 1.0e+6;
  Mp->strcomp = 1;
/*
 End of Hinkley modification
*/


  // compute and print values initial values
  compute_req_val (lcid);
  print_init(-1, "wt");
  print_step(lcid, 0, 0, fc);
  print_flush();


  //  assembling of tangent stiffness matrix
  stiffness_matrix (lcid);
  // norm of load vector
  norfa=sizev(fc,n);
  //  internal iteration loop
  for (j=0;j<nii;j++)
  {
    //  computation of internal forces
    internal_forces (lcid,fi);
    //  vector of unbalanced forces
    for (k=0;k<n;k++)
      fb[k]=fc[k]-fi[k];
    //  norm of vector of unbalanced forces
    norf=sizev(fb,n);
    norf /= norfa;
    
    if (norf<err)
    {
      Mm->updateipval ();
      compute_req_val (lcid);
      print_step(lcid, j+1, -0.0000001, fc);
      print_flush();
      break;
    }
    //Smat->solve_system (Gtm,dr,fb);
    Mp->ssle->solve_system (Gtm,Smat,dr,fb,Out);

/*
    double tmp1, tmp2, tmp3, tmp4;
    long ncomp;
    tmp1=tmp2=tmp3=tmp4=0.0;
    fprintf(Out, "\n\n*Krok %ld\n", j);
    fprintf(Out,"\n\nn\tfc\tfi\tfc-fi\tr\tr+dr\tddr\n");
    for(k=0; k<n; k++)
    {
      tmp1+=fc[k]*fc[k];
      tmp2+=fi[k]*fi[k];
      tmp3+=r[k]*r[k];
      r[k]+=dr[k];
      tmp4+=r[k]*r[k];
      if (k%100==0)
        fprintf(Out, "%ld\t%e\t%e\t%e\t%e\t%e\t%e\n",k, tmp1, tmp2, tmp1-tmp2, tmp3, tmp4, tmp3-tmp4);
    }
    fprintf(Out, "%ld\t%e\t%e\t%e\t%e\t%e\t%e\n",k, tmp1, tmp2, tmp1-tmp2, tmp3, tmp4, tmp3-tmp4);
    fprintf(Out,"\n\nIpp\t omega\n");
   
    for (k=0; k < Mm->tnip; k++)
    {
      if (Mm->ip[k].ncompother > 8)
      {
        ncomp = 2*Mm->ip[k].ncompstr;
        fprintf(Out, "%ld\t%e\t%e\n", k, Mm->ip[k].other[ncomp],Mm->ip[k].other[ncomp+1]);
      }
    }
    fflush(Out);*/
    for(k=0; k<n; k++)
      r[k]+=dr[k];
/*    swap_other();
    compute_req_val (lcid);
    print_step(lcid, j+1, j+1, fc);
    print_flush();
    swap_other();*/
    fprintf (stdout,"\n Equilibrium interation loop j %ld     norf=%e", j, norf);
  }
  if ((nii > 0) && (norf > err))
  {
    fprintf(stdout, "\n Initial equilibrium could not be reached\n");
    fprintf(stdout, " Program will be terminated\n");
    print_close();
    return;
  }
  else
    fprintf(stdout, "\n\n Starting incremental loading . . .\n");


  // ***************************
  //  main iteration loop  ****
  // ***************************
  for (i=0;i<ni;i++){
    
    //  backup of left hand side vector
    for (j=0;j<n;j++){
      rb[j]=r[j];
    }
    //  backup of reached lambda parameter
    blambda=lambda;
   
    fprintf (stdout,"\n increment %ld   lambda %e,    dlambda %e",i,lambda,dlambda);
    
    //  vector of maximum load and vector of load increment
    for (j=0;j<n;j++){
      f[j]=fc[j]+lambda*fp[j];
      fb[j]=dlambda*fp[j];
    }
    
    //  assembling of tangent stiffness matrix
    if ((Mp->nlman.stmat==tangent_stiff) || (i == 0))
      stiffness_matrix (lcid);

    //  solution of K(r).v=F

    //Smat->solve_system (Gtm,dr,fb);
    Mp->ssle->solve_system (Gtm,Smat,dr,fb,Out);

    for (j=0;j<n;j++){
      r[j]+=dr[j];
    }
    //  computation of internal forces
    internal_forces (lcid,fi);
    //  vector of unbalanced forces
    for (j=0;j<n;j++){
      fb[j]=f[j]+dlambda*fp[j];
    }
    norfa=sizev(fb,n);
    for (j=0;j<n;j++){
      fb[j] -= fi[j];
    }
    //  norm of vector of unbalanced forces
    norf=sizev(fb,n);
    norf /= norfa;
    
    
    
//    fprintf (Out,"\n norf=%e",norf);

    if (norf<err){
      lambda+=dlambda;
      Mm->updateipval ();
      compute_req_val (lcid);
      print_step(lcid, i+1, lambda, f);
      print_flush();
      continue;
    }
    
    //  internal iteration loop
    for (j=0;j<ini;j++){
      
      //  solution of K(r).v=F
      //Smat->solve_system (Gtm,dr,fb);
      Mp->ssle->solve_system (Gtm,Smat,dr,fb,Out);

      for (k=0;k<n;k++){
	r[k]+=dr[k];
      }
      
      //  computation of internal forces
      internal_forces (lcid,fi);
      
      //  vector of unbalanced forces
      for (k=0;k<n;k++){
	fb[k]=f[k]+dlambda*fp[k]-fi[k];
      }
      
      //  norm of vector of unbalanced forces
      norf=sizev(fb,n);
      norf /= norfa;
      
      fprintf (stdout,"\n increment %ld   inner loop j %ld     norf=%e  dlambda %e",i,j,norf,dlambda);
//      fprintf (Out,"\n j=%ld     norf=%e",j,norf);
      
      if (norf<err)  break;
    }
    
    if (j==ini || norf>err){
      dlambda/=2.0;
      if (dlambda<dlambdamin)  dlambda=dlambdamin;

      if (Mespr==1){
	fprintf (stdout,"\n increment must be modified (dlambda decrease) dlambda=%e    norf=%e",dlambda,norf);
//	fprintf (Out,"\n increment must be modified (dlambda decrease) dlambda=%e    norf=%e",dlambda,norf);
      }
      
      for (j=0;j<n;j++){
      	r[j]=rb[j];
      }
      lambda=blambda;
    }
    else{
      lambda+=dlambda;
      Mm->updateipval ();
      compute_req_val (lcid);
      print_step(lcid, i+1, lambda, f);
      print_flush();
    }
    
  }
  
  delete [] dr;  delete [] fi;  delete [] fb;  delete [] f;
  print_close();
}



/**
  The function saves actual step of arclength method to the text file.

  @param fi - id of saved step
  @param n - number of r components
  @param blambda - attained load coefficient lambda
  @param dl - legth of arc
  @param r - %vector of attained displacements
  @param fp - %vector of proportional load

  @return The function does not return anything.

  Created by JK,
*/
void arclsave (long fi,long n,double blambda,double dl,double *r,double *fp)
{
  long i,j;
  char *file;
  FILE *aux;
  file = new char[strlen(Mp->path)+11+1];
  sprintf (file,"%sarcl.backup",Mp->path);
  aux = fopen (file,"w");

  //aux = fopen ("arcl.backup","w");
  
  //  attained number of steps
  fprintf (aux,"%ld\n",fi);
  //  number of rows
  fprintf (aux,"%ld\n",n);
  //  attained coefficient of proportionality
  fprintf (aux,"%e\n",blambda);
  //  arc-length
  fprintf (aux,"%e\n",dl);
  //  number of selected dofs
  fprintf (aux,"%ld\n",Mp->nlman.nsdofal);
  //  selected dofs
  for (i=0;i<Mp->nlman.nsdofal;i++){
    fprintf (aux,"%ld\n",Mp->nlman.seldofal[i]);
  }
  //  left hand side vector
  for (i=0;i<n;i++){
    fprintf (aux,"%e\n",r[i]);
  }
  //  proportionality vector
  for (i=0;i<n;i++){
    fprintf (aux,"%e\n",fp[i]);
  }
  //  number of integration points
  fprintf (aux,"%ld\n",Mm->tnip);
  //  integration points
  for (i=0;i<Mm->tnip;i++){
    fprintf (aux,"%ld %ld\n",Mm->ip[i].ncompstr,Mm->ip[i].ncompother);
    for (j=0;j<Mm->ip[i].ncompstr;j++){
      fprintf (aux,"%e\n",Mm->ip[i].strain[j]);
    }
    for (j=0;j<Mm->ip[i].ncompstr;j++){
      fprintf (aux,"%e\n",Mm->ip[i].stress[j]);
    }
    for (j=0;j<Mm->ip[i].ncompother;j++){
      fprintf (aux,"%e\n",Mm->ip[i].eqother[j]);
    }
  }
  
  fclose (aux);
}



/**
  The function restores saved step of arclength method from the text file.

  @param fi - id of saved step
  @param n - number of r components
  @param blambda - attained load coefficient lambda
  @param dl - legth of arc (output)
  @param r - %vector of attained displacements (output)
  @param fp - %vector of proportional load (output)

  @return The function returns restored values in the passed parameters.

  Created by JK,
*/
void arclopen (long &fi,long &n,double &blambda,double &dl,double *r,double *fp)
{
  long i,j;
  char *file;
  FILE *aux;

  file = new char[strlen(Mp->path)+11+1];
  sprintf (file,"%sarcl.backup",Mp->path);
  aux = fopen (file,"r");
  
  //  attained number of steps
  fscanf (aux,"%ld",&fi);
  //  number of rows of the matrix = number of unknowns
  fscanf (aux,"%ld",&n);
  //  attained coefficient of proportionality
  fscanf (aux,"%le",&blambda);
  //  arc-length
  fscanf (aux,"%le",&dl);
  //  number of selected dofs
  fscanf (aux,"%ld",&Mp->nlman.nsdofal);
  //  selected dofs
  for (i=0;i<Mp->nlman.nsdofal;i++){
    fscanf (aux,"%ld",&Mp->nlman.seldofal[i]);
  }
  //  left hand side vector
  for (i=0;i<n;i++){
    fscanf (aux,"%le",&r[i]);
  }
  //  proportionality vector
  for (i=0;i<n;i++){
    fscanf (aux,"%le",&fp[i]);
  }
  //  number of integration points
  fscanf (aux,"%ld",&Mm->tnip);
  //  integration points
  for (i=0;i<Mm->tnip;i++){
    fscanf (aux,"%ld %ld",&Mm->ip[i].ncompstr,&Mm->ip[i].ncompother);
    for (j=0;j<Mm->ip[i].ncompstr;j++){
      fscanf (aux,"%le",&Mm->ip[i].strain[j]);
    }
    for (j=0;j<Mm->ip[i].ncompstr;j++){
      fscanf (aux,"%le",&Mm->ip[i].stress[j]);
    }
    for (j=0;j<Mm->ip[i].ncompother;j++){
      fscanf (aux,"%le",&Mm->ip[i].eqother[j]);
    }
  }
  
  fclose (aux);
}



/**
  ???!!! candidate for removal
*/
void print_output(long lcid)
{
/*
  //  strains
  if (Mp->straincomp==1){
    Mm->computestrains (lcid);
    Mm->stra.transformvalues (1);
  }
    
  //  stresses
  if (Mp->stresscomp==1){
    Mm->computenlstresses (lcid);
    Mm->stre.transformvalues(0);
  }
  
  //  reactions
  if (Mp->reactcomp==1)
    Mb->lc[lcid].compute_reactions (lcid);
  
    
  //  output data
  Lsrs->output (Out,lcid);    
  // other at the integration points  
  if ((Mp->straincomp==1) || (Mp->stresscomp==1))
    print_other (Out,lcid);
*/
}



/**

   created  1.12.2002, Ladislav Svoboda, termit@cml.fsv.cvut.cz
 */
void arclsave_adapt (long fi,double lambda,double dlambda,double dl)
{
  char *file;
  FILE *aux;
  
  file = new char[strlen(Mp->path)+19+1];
  //  sprintf (file,"%sarcl.backup.ada.%03ld",Mp->path,Mp->nial);
  sprintf (file,"%sarcl.backup.ada",Mp->path);
  aux = fopen (file,"w");
  
  //  attained number of steps
  fprintf (aux,"%ld\n",fi);
  
  //  attained coefficient of proportionality
  fprintf (aux,"%.16e\n",lambda);
  
  //  
  fprintf (aux,"%.16e\n",dlambda);
  
  //  arc-length
  fprintf (aux,"%.16e\n",dl);
  
  fclose (aux);
}
/**

   created  1.12.2002, Ladislav Svoboda, termit@cml.fsv.cvut.cz
 */
void arclopen_adapt (long &fi,double &lambda,double &dlambda,double &dl)
{
  char *file;
  FILE *aux;
  
  file = new char[strlen(Mp->path)+19+1];
  //  sprintf (file,"%sarcl.backup.ada.%03ld",Mp->path,Mp->nial);
  sprintf (file,"%sarcl.backup.ada",Mp->path);
  aux = fopen (file,"r");
  
  //  attained number of steps
  fscanf (aux,"%ld",&fi);
  
  //  attained coefficient of proportionality
  fscanf (aux,"%le",&lambda);
  
  //  
  fscanf (aux,"%le",&dlambda);
  
  //  arc-length
  fscanf (aux,"%le",&dl);

  fclose (aux);
}

/**

   created  1.12.2002, Ladislav Svoboda, termit@cml.fsv.cvut.cz
 */
void newmeshgen (int w,long i)
{
  char procname[255];
  
  if (Mespr==1)  fprintf (stdout,"\n\n ***  T3D generator - start  ***\n");
  
  if (Mespr) sprintf (procname,"/home/dr/Bin/T3d -i %sXmesht3d.in -o %smesh.out.%0*ld -m %smesh.bgm.%0*ld -d 100 -p 8 -r 1 -k %ld"
	                      ,Mp->path,Mp->path,w,i,Mp->path,w,i-1,Mt->give_degree(0));
  else       sprintf (procname,"/home/dr/Bin/T3d -i %sXmesht3d.in -o %smesh.out.%0*ld -m %smesh.bgm.%0*ld -d 100 -p 8 -r 1 -k %ld  > /dev/null 2>&1"
	                      ,Mp->path,Mp->path,w,i,Mp->path,w,i-1,Mt->give_degree(0));
  
  if (Mespr==1)  fprintf (stdout,"\n\n ***  T3D generator - end  *****\n");


  system (procname);
  
  
  sprintf (procname,"sed  -e s/out.i/out.%0*ld/g  -e s/dat.i/dat.%d/g  %sprep.pre > %sprep.%0*ld",w,i,0,Mp->path,Mp->path,w,i);
  system (procname);
  
  if (Mespr)
    sprintf (procname,"./mechprep %sprep.%0*ld %ssifel.in.%0*ld",Mp->path,w,i,Mp->path,w,i);
  else
    sprintf (procname,"./mechprep %sprep.%0*ld %ssifel.in.%0*ld  > /dev/null 2>&1",Mp->path,w,i,Mp->path,w,i);

  system (procname);
  

  sprintf (procname,"rm -f %sprep.%0*ld",Mp->path,w,i);
  system (procname);
}

/*
      FILE *ff;
      ff = fopen ("problemnl/aux.txt","a");      if (ff==NULL) fprintf (stderr,"\n ff file has not been opened.\n\n");
      for (long g=0;g<Ndofn;g++)
	fprintf (ff,"%40.20lf\n",Lsrs->lhs[i]);
      fclose (ff);
      
      


*/

/**
   function solves system of nonlinear algebraic equation
   by arc-length method (continuation method)
   only one right hand side vector is supported with respect
   to nonlinearity and absence of superposition

   d stands for delta
   dd stands for capital delta
   
   fc - nonproportional %vector
   fp - proportional %vector
   n - order of the system
   
   @param lcid - load case id
   
   23.12.2004
*/
long arclengthadapt (long lcid,long adaptcontrol)
{
  long i,j,k,n,ni,ini,stop,modif,li,newmesh, numr;
  double a0,a1,a2,l1,l2,dl,dlmax,dlmin,psi,lambda,blambda,dlambda,ddlambda,norf,norfp,norv,norfa,zero,ierr;
  double lambdao,ss1,ss2,ss3,ss4,ss5;
  double *r,*ra,*ddr,*u,*v,*f,*fa,*fp,*fc,*fi;
  
  
  
  //  number of rows of the matrix
  n = Ndofm;
  //  maximum number of increments
  ni = Mp->nlman.nial;
  //  maximum number of iterations in one increment
  ini = Mp->nlman.niilal;
  //  computer zero
  zero = Mp->zero;
  //  required error in inner loop
  ierr = Mp->nlman.erral;
  //  length of arc
  dl = Mp->nlman.dlal;
  //  maximum length of arc
  dlmax = Mp->nlman.dlmaxal;
  //  minimum length of arc
  dlmin = Mp->nlman.dlminal;
  //  displacement-loading driving switch
  psi = Mp->nlman.psial;
  
  dlambda=0.0;
  
  //  allocation of auxiliary arrays
  r = new double [n];
  ddr = new double [n];
  u = new double [n];
  v = new double [n];
  f = new double [n];
  fa = new double [n];
  fi = new double [n];
  
  
  //  initialization phase
  ra = Lsrs->give_lhs (lcid);
  fp = Lsrs->give_rhs (lcid*2);
  fc = Lsrs->give_rhs (lcid*2+1);
  
  if (Mp->nlman.hdbackupal == 2){            // termit
    arclopen (li,n,lambda,dl,ra,fp);
  }
  else if (adaptcontrol >= 2){
    arclopen_adapt (li,lambda,blambda,dl);
  }
  else{
    lambda=0.0;
    lambdao=0.0;
    li=0;
  }
  
  //  norm of proportionality vector
  norfp = loadincr(fp,n);
  modif=0;


  // assembling reached load vector
  for (j=0;j<n;j++){
    fa[j]=fc[j]+(lambda+dlambda)*fp[j];
    f[j]=fp[j];
  }
  if (li)
    print_init(-1, "at");
  else
  {
    print_init(-1, "wt");
    print_step(lcid, li, lambda, fa);
  }
  
  
  //  elastic trial response of structure
  Mp->phase=1;
  //  stiffness matrix based on elastic models
  stiffness_matrix (lcid);
  //  solution of K.r=f
  //Smat->solve_system (Gtm,ra,f);
  Mp->ssle->solve_system (Gtm,Smat,ra,f,Out);

  computestresses (0);

  //  switch to the special material models
  Mp->phase=2;
  
  
  // ***************************
  //  main iteration loop  ****
  // ***************************
  for (i=li;i<ni;i++){
    stop=1; // termit
    //  backup of left hand side vector
    copyv (ra, r, n);
    //  backup of reached lambda parameter
    blambda=lambda;

    
    //  assembling of tangent stiffness matrix
    if ((Mp->nlman.stmat==tangent_stiff) || (i == li))
      stiffness_matrix (lcid);
    
    //  backup of the fp, in ldl_sky the right hand side will be destroyed
    copyv(fp, f, n);

    //  solution of K(r).v=F
    //Smat->solve_system (Gtm,v,f);
    Mp->ssle->solve_system (Gtm,Smat,v,f,Out);

    //  generalized norm of displacement increments
    norv = displincr (v,n);

    //  compute new dlambda increment
    dlambda = dl/sqrt(norv+psi*psi*norfp*norfp);
    
    for (j=0;j<n;j++){
      ddr[j]=dlambda*v[j];
      ra[j]+=ddr[j];
      fa[j]=fc[j]+(lambda+dlambda)*fp[j];
    }
    ddlambda=dlambda;
    
    //  computation of internal forces
    //Mm->computestresses (0);
    internal_forces (lcid,fi);
    subv(fa, fi, f, n);    
    norf=sizev(f,n);
    norfa = sizev(fa, n);
    norf /= norfa;

    
    //if (Mespr==1)  fprintf (stdout,"\n ddlambda %e    dlambda %e   norf %e  ierr %e",ddlambda,dlambda,norf,ierr);
    if (Mespr==1)  fprintf (stdout,"\n increment %ld     norv %e      norf %e",i,norv,norf);
    
    if (norf<ierr){
      // ******************************************
      //  no inner iteration loop is required  ***
      // ******************************************
      modif++;

      if (modif>1){
	//  arc length modification
	dl*=2.0;
	if (dl>dlmax)  
          dl=dlmax;
	modif=0;
	if (Mespr==1){
	  fprintf (stdout,"\n arc-length must be modified (dl increase) dl=%e",dl);
//	  fprintf (Out,"\n arc-length must be modified (dl increase) dl=%e",dl);
	}
      }
      lambda+=dlambda;
      Mm->updateipval ();
      print_step(lcid, i+1, lambda, fa);
      print_flush();
    }
    else{
      // ****************************
      //  inner iteration loop  ****
      // ****************************
      stop=0;
      for (j=0;j<ini;j++){
	
	if (Mp->nlman.stmat==tangent_stiff)
	  stiffness_matrix (lcid);
	
	//  back substitution
	//Smat->solve_system (Gtm,u,f);
	Mp->ssle->solve_system (Gtm,Smat,u,f,Out);

        copyv(ddr, f, n);
        addv(ddr, u, n);
	//  coefficient of quadratic equation
	quadeqcoeff (ddr,v,n,ddlambda,psi,norfp,dl,a0,a1,a2);
	/*
	  fprintf (Out,"\n\n\n Kontrola kvadraticke rovnice");
	  fprintf (Out,"\n norfp     %15.10le",norfp);
	  fprintf (Out,"\n norv      %15.10le",norv);
	  fprintf (Out,"\n (ddr,v)   %15.10le",ss(ddr,v,n));
	  fprintf (Out,"\n (ddr,ddr) %15.10le",ss(ddr,ddr,n));
	  fprintf (Out,"\n dl        %15.10le",dl);
	  fprintf (Out,"\n ddlambda  %15.10le",ddlambda);
	  fprintf (Out,"\n psi       %15.10le",psi);
	  fprintf (Out,"\n a2        %15.10le",a2);
	  fprintf (Out,"\n a1        %15.10le",a1);
	  fprintf (Out,"\n a0        %15.10le",a0);
	  fprintf (Out,"\n discrim   %15.10le",a1*a1-4.0*a2*a0);
	  fprintf (Out,"\n\n");
	*/
	
	//  solution of quadratic equation
        numr = solv_polynom_2(a2, a1, a0, l1, l2);
        switch (numr)
	{
	  case -1:
            fprintf (stderr,"\n\n infinite number of solution of constrained condition in function arclength");
            abort();
            break;
	  case 0:
            fprintf (stderr,"\n\n nonsolvable constrained condition in function arclength");
            abort();
            break;
    	  case 1:
            dlambda = l1;
            break;
	  default:
            break;
	}
	ss1=0.0;  ss2=0.0;
	ss3=0.0;  ss4=0.0;  ss5=0.0;
	for (k=0;k<n;k++){
	  ss1+=(ddr[k]+l1*v[k])*f[k];
	  ss2+=(ddr[k]+l2*v[k])*f[k];
	  ss3+=(ddr[k]+l1*v[k])*(ddr[k]+l1*v[k]);
	  ss4+=(ddr[k]+l2*v[k])*(ddr[k]+l2*v[k]);
	  ss5+=f[k]*f[k];
	}
	
	/*
	if (ss1/ss3/ss5>ss2/ss4/ss5)  
          dlambda=l1;
	else                          
          dlambda=l2;
	*/
	if (fabs(l1)>fabs(l2))  dlambda=l2;
	else dlambda=l1;
	
	
	//fprintf (stdout,"\n increment  %ld     inner loop  %ld   x1=%e x2=%e   dlambda %e",i,j,l1,l2,dlambda);
	//fprintf (stdout,"\n increment  %ld     inner loop  %ld   x1=%e x2=%e   ddlambda %e",i,j,l1,l2,ddlambda);

	for (k=0;k<n;k++){
	  ddr[k]+=dlambda*v[k];
	  ra[k]+=u[k]+dlambda*v[k];
	  fa[k]+=dlambda*fp[k];
	}
	ddlambda+=dlambda;
	
	//fprintf (stdout,"   ddlambda %e",ddlambda);
	
	//fprintf (Out,"\n ddlambda %e     dlambda %e",ddlambda,dlambda);
	//  computation of internal forces
	internal_forces (lcid,fi);
        subv(fa, fi, f, n);	
	norf=sizev(f,n);
	norfa = sizev(fa, n);
	norf /= norfa;

	if (Mespr==1){
	  fprintf (stdout,"\n    norf=%e  ierr=%e",norf,ierr);
	  //fprintf (Out,"\n increment  %ld     inner loop  %ld    norf=%e",i,j,norf);
	}
	if (norf<ierr){
	  lambda+=ddlambda;
	  Mm->updateipval ();
	  stop=1; 
          print_step(lcid, i+1, lambda, fa);
          print_flush();
	  break;
	}
      }
      modif=0;
      if (stop==0){
	//  modification of the arc length
	dl/=2.0;
	if (dl<dlmin){
          dl=dlmin;  
          break; 
        }
	if (Mespr==1){
	  fprintf (stdout,"\n arc length must be modified (dl decrease) dl=%e",dl);
	  //fprintf (Out,"\n arc length must be modified (dl decrease) dl=%e",dl);
	}
	//  restoring of left hand side vector
        copyv(r, ra, n);
	//  restoring of lambda parameter
	lambda=blambda;
      }
    }
    
    fprintf (stdout,"\n increment %ld    total lambda %e",i,lambda);

    if (stop==0)
      continue;
    
    if (Mp->adaptflag & 16 && i%10 && Mm->plast)
      continue;
    
    newmesh = 0;
    if (Mp->adaptivity && Mp->nlman.nial > (i+1) && !( (Mp->adaptflag & 16) && (Mp->nlman.nial==100 || Mp->nlman.nial==1))){
      
      // print of fxlambda second part is in adaptivity.cpp
      //FILE *ff;
      //sprintf (file,"%sfxlambda.dat",Mp->path);
      //ff = fopen (file,"a");
      //if (ff==NULL) fprintf (stderr,"\n ff file has not been opened.\n\n");
      //fprintf (ff,"  %20.10f",lambda);
      //fclose (ff);
      
      long width = 1 + (long)log10((double)Mp->nlman.nial);
      newmesh = Ada->run (1,width,i);
    }
    
    if (newmesh){
      i++;             // toto se da odstranit
      break;
    }
    // timret
    //print_output(lcid);
  }
  
  // ------------------------------------
  //  finish of main iteration loop  ----
  // ------------------------------------
  
  if (Mp->nlman.hdbackupal==1)
    arclsave (i,n,lambda,dl,ra,fp);

  // termit
  if (Mp->adaptivity)
  {
    if ((Mp->adaptflag & 16) && (Mp->nlman.nial==100 || Mp->nlman.nial==1))
      Ada->run (0,0,0);  // only for testing
    else
    {
      if (i < ni)
	arclsave_adapt (i,lambda,0,dl);
    }
  }
  
  //if (Mp->adaptivity)   arclsave_adapt (i,blambda,ddlambda,dl);
  // timret
  print_close();
  delete [] r;		    
  delete [] fi;  
  delete [] fa;  
  delete [] f;
  delete [] v;  
  delete [] u;  
  delete [] ddr;
  
  return (i);
}


void refresh_plast ()
{
  for (long i=0;i<Mm->tnip;i++)
    if ( Mm->j2f[Mm->ip[i].idm[0]].give_consparam (i,0) ){
      Mm->plast = 1;
      return;
    }
}

/**
   if adaptcontrol =
   1 - arclength nyni bezi poprve
   2 - arclength nyni nebezi poprve
   
   created  1.12.2002, Ladislav Svoboda, termit@cml.fsv.cvut.cz
 */
void nonlinear_solver_adaptiv (long lcid)
{
  int width;
  long i,adaptcontrol;
  char file[255];
  
  switch (Mp->nlman.tnlinsol){
  case arcl:{
    if (Mespr==1){
      fprintf (stdout,"\n\n ----------------------------");
      fprintf (stdout,  "\n ***  Arc-length - start  ***\n");
    }
    
    width = 1 + (long)log10((double)Mp->nlman.nial);
    adaptcontrol = 1;
    
    for (i=0; i < Mp->nlman.nial ;){
      // i = number of performed steps
      //i = arclength (lcid,adaptcontrol);
      
      if ((i<Mp->nlman.nial) || ((Mp->nlman.nial==1) && (Mp->adaptflag & 16))){
	if (Mespr==1)  fprintf (stdout,"\n\n NEW MESH will be generated\n");
	
	newmeshgen (width,i);
	sprintf (file,"%s%s.in.%0*ld",Mp->path,Mp->filename,width,i);
	newmeshread (file,lcid);
	
	if ( i<Mp->nlman.nial ){
	  if (Mm->plast == 0)
	    refresh_plast ();
	  
	  arclength15  (lcid);
	}
	else
	  Ada->run (4,width,i); // only for testing
      }
      
      //if (!(Mp->adaptflag & 16)){  sprintf (file,"%s%s.dx.%ld.1",Mp->path,Mp->filename,i);  print_default_2_dx (Gtm,Mp,Mt,0,file);  }
      
      adaptcontrol++;
    }
    
    if (Mespr==1){
      fprintf (stdout,"\n\n ***   Arc-length is finished  ***");
      fprintf (stdout,  "\n *********************************");
    }
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown solver of nonlinear equation system is required");
    fprintf (stderr,"\n in function nonlinear_solver_adaptiv (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
}


long arclength15 (long lcid)
{
  long j,k,n,ini,stop,li;
  double a0,a1,a2,dl,dlmax,dlmin,psi,lambda,blambda,bdlambda,dlambda,ddlambda,norf,norfp,norv,zero,ierr;
//  double l1, l2;
//  double ss1,ss2,ss3,ss4,ss5;
  double *r,*ra,*ddr,*u,*v,*f,*fa,*fp,*fc,*fi;
  //  FILE *gr;
  //  char file[255];
  // double d;


  //  gr = fopen (Mp->auxfile,"a");
  
  n = Ndofm;                   
  ini = Mp->nlman.niilal;           
  zero = Mp->zero;          
  ierr = Mp->nlman.erral;           
  dl = Mp->nlman.dlal;              
  dlmax = Mp->nlman.dlmaxal;        
  dlmin = Mp->nlman.dlminal;        
  psi = Mp->nlman.psial;          
  
  r = new double [n];     ddr = new double [n];   u = new double [n];     v = new double [n];   
  f = new double [n];     fa = new double [n];    fi = new double [n];  
  
  //  Mm->alloc_backup (); 
  ra = Lsrs->give_lhs (lcid);
  fp = Lsrs->give_rhs (lcid*2);
  fc = Lsrs->give_rhs (lcid*2+1);
  
  arclopen_adapt (li,lambda,dlambda,dl);
  
  //  aux_mech_nonlin_print (gr,ra,lambda);
  // *******************************************************************
  //  backup of left hand side vector
  fprintf (stdout,"\n arc-length15: lambda %e  dl %e",lambda,dl);
  
  stiffness_matrix (lcid);
  for (j=0;j<n;j++)    f[j]=fp[j];

  //Smat->solve_system (Gtm,v,f);
  Mp->ssle->solve_system (Gtm,Smat,v,f,Out);
  norv = displincr (v,n);
  norfp = loadincr(fp,n);
    
  if (0 && !Mm->plast){
    for (j=0;j<n;j++)
      ra[j]=(lambda+dlambda)*v[j];
    
    //    aux_mech_nonlin_print (gr,ra,lambda);
  }
  else{  
    stop = 0;

    for (j=0;j<n;j++){      r[j]=ra[j];    }
    blambda=lambda;
    bdlambda=dlambda;
    //    Mm->backup_state_var ();
    
    for (j=0;j<n;j++){
      ddr[j]=dlambda*v[j];
      fa[j]=fc[j]+(lambda+dlambda)*fp[j];
    }
    ddlambda=dlambda;
    
    internal_forces (lcid,fi);
    
    for (k=0;k<n;k++)
      f[k]=fa[k]-fi[k];
    
    norf=ss(f,f,n);
    norf=sqrt(norf);    if (Mespr==1)  fprintf (stdout,"\n %e %e norf %e",lambda,dl,norf);
    
    if (norf<ierr){
      lambda+=dlambda;
      Mm->updateipval ();
      stop = 1;
    }

    while (!stop){


      
      // ****************************
      //  inner iteration loop  ****   arclength15
      // ****************************
      for (j=0;j<10*ini;j++){
	
      //      aux_mech_nonlin_print (gr,ra,lambda);
      //  back substitution
	//Smat->solve_system (Gtm,u,f);
	Mp->ssle->solve_system (Gtm,Smat,u,f,Out);

	for (k=0;k<n;k++){
	  f[k]=ddr[k];
	  ddr[k]+=u[k];
	}
	
	//  coefficient of quadratic equation
	//a2=norv+psi*psi*norfp;
	//a1=2.0*ss(ddr,v,n)+2.0*ddlambda*psi*psi*norfp;
	//a0=ss(ddr,ddr,n)+ddlambda*ddlambda*psi*psi*norfp-dl*dl;
	
	quadeqcoeff (ddr,v,n,ddlambda,psi,norfp,dl,a0,a1,a2);
	
	
	//  solution of quadratic equation
	/*
	if (fabs(a2)<zero){
	  if (fabs(a1)<zero){
	    if (fabs(a0)>zero)  fprintf (stderr,"\n\n nonsolvable constrained condition in function arclength");
	  else  fprintf (stderr,"\n\n infinite number of solution of constrained condition in function arclength");
	  }
	  else{
	    dlambda=(0.0-a0)/a1;
	  }
	}
	else{
	  d=a1*a1-4.0*a2*a0;
	  
	  if (d<0.0){
	    fprintf (stderr,"\n negative discriminant in function arclength");
	    break;
	  }
	  
	  
	  l1=(0.0-a1+sqrt(d))/2.0/a2;
	  l2=(0.0-a1-sqrt(d))/2.0/a2;
	  
	  if (l1>l2)  dlambda=l1;
	  else  dlambda=l2;
	  \\if (fabs(l1)<fabs(l2))  dlambda=l1;
	  \\else  dlambda=l2;
	}
	*/
	//  zacatek novinky   arclength15
        /*
	ss1=0.0;  ss2=0.0;
	ss3=0.0;  ss4=0.0;  ss5=0.0;
	for (k=0;k<n;k++){
	  ss1+=(ddr[k]+l1*v[k])*f[k];
	  ss2+=(ddr[k]+l2*v[k])*f[k];
	  ss3+=(ddr[k]+l1*v[k])*(ddr[k]+l1*v[k]);
	  ss4+=(ddr[k]+l2*v[k])*(ddr[k]+l2*v[k]);
	  ss5+=f[k]*f[k];
	}
       
	if (ss1/ss3/ss5>ss2/ss4/ss5)  dlambda=l1;
	else          dlambda=l2;
        */
	//  konec novinky
	
        dlambda = 0;
	
	for (k=0;k<n;k++){
	  ddr[k]+=dlambda*v[k];
	  ra[k]+=u[k]+dlambda*v[k];
	  fa[k]+=dlambda*fp[k];
	}
	ddlambda+=dlambda;
	
	internal_forces (lcid,fi);
	
	for (k=0;k<n;k++)
	  f[k]=fa[k]-fi[k];
	
	norf=ss(f,f,n);   norf=sqrt(norf);
	
	if (Mespr==1){ 
          fprintf (stdout,"\n increment  %ld     inner loop  %ld    norf=%e",li,j,norf); 
//          fprintf (Out,"\n increment  %ld     inner loop  %ld    norf=%e",li,j,norf);
        }
	
	if (norf<0.10*ierr){
	  lambda+=ddlambda;
	  //	  aux_mech_nonlin_print (gr,ra,lambda);
	  Mm->updateipval ();
	  stop=1;  break;
	}
      }   // arclength15
      
      if (stop==0){
	//  modification of the arc length
	dl/=2.0;
	if (dl<dlmin){  dl=dlmin;  break; }
	
	if (Mespr==1){
	  fprintf (stdout,"\n arc length must be modified (dl decrease) dl=%e    norf=%e",dl,norf);
//	  fprintf (Out,"\n arc length must be modified (dl decrease) dl=%e    norf=%e",dl,norf);
	}
	//  restoring of left hand side vector
	for (j=0;j<n;j++){
	  ra[j]=r[j];
	}
	//  restoring of lambda parameter
	lambda=blambda;
	ddlambda=bdlambda;
	//  restoring of state variables
	//	Mm->restore_state_var ();

	//****************
	stop=1;
	//****************

      }
    }
  }
  
  // ------------------------------------
  //  finish of main iteration loop  ----  arclength15
  // ------------------------------------
  
  arclsave_adapt (li,lambda,0.0,dl);
  
  delete [] r;
  delete [] fi;  
  delete [] fa;  
  delete [] f;
  delete [] v;  
  delete [] u;  
  delete [] ddr;
  
  //  fclose (gr); 
  
  return (li);
} //arclength15

