#include "plelemsubqt.h"
#include "global.h"
#include "globmat.h"
#include "genfile.h"
#include "adaptivity.h"
#include "node.h"
#include "element.h"
#include "intpoints.h"
#include "plelemlt.h"
#include "plelemlq.h"
#include "plelemqq.h"
#include <stdlib.h>
#include <math.h>



planeelemsubqt::planeelemsubqt (void)
{
  long i,j;

  //  number nodes on element
  nne=6;
  //  number of DOFs on element
  ndofe=12;
  //  number of strain/stress components
  tncomp=3;
  //  number of components for graphic purposes
  gncomp=4;
  //  number of functions approximated
  napfun=2;
  //  order of numerical integration of mass matrix
  intordmm=6;
  //  number of edges on element
  ned=3;
  //  number of nodes on one edge
  nned=3;
  //  order of numerical integration on element edges (boundaries)
  intordb = 3;

  //  number of blocks (parts of geometric matrix)
  nb=2;

  ncomp = new long [nb];
  ncomp[0]=2;
  ncomp[1]=1;

  cncomp = new long [nb];
  cncomp[0]=0;
  cncomp[1]=2;

  nip = new long* [nb];
  intordsm = new long* [nb];
  for (i=0;i<nb;i++){
    nip[i] = new long [nb];
    intordsm[i] = new long [nb];
  }
  
  nip[0][0]=3;  nip[0][1]=0;
  nip[1][0]=0;  nip[1][1]=3;
  
  intordsm[0][0]=3;  intordsm[0][1]=0;
  intordsm[1][0]=0;  intordsm[1][1]=3;

  tnip=0;
  for (i=0;i<nb;i++){
    for (j=0;j<nb;j++){
      tnip+=nip[i][j];
    }
  }
}

planeelemsubqt::~planeelemsubqt (void)
{
  long i;
  
  for (i=0;i<nb;i++){
    delete [] nip[i];
    delete [] intordsm[i];
  }
  delete [] intordsm;
  delete [] nip;
  delete [] ncomp;
  delete [] cncomp;
  
}

void planeelemsubqt::eleminit (long eid)
{
  long ii,jj;
  Mt->elements[eid].nb=nb;
  Mt->elements[eid].intordsm = new long* [nb];
  Mt->elements[eid].nip = new long* [nb];

  for (ii=0;ii<nb;ii++){
    Mt->elements[eid].intordsm[ii] = new long [nb];
    Mt->elements[eid].nip[ii] = new long [nb];
    for (jj=0;jj<nb;jj++){
      Mt->elements[eid].intordsm[ii][jj]=intordsm[ii][jj];
      Mt->elements[eid].nip[ii][jj]=nip[ii][jj];
    }
  }
}

/**
   function approximates function defined by nodal values

   @param xi,eta - natural coordinates
   @param nodval - nodal values
*/
double planeelemsubqt::approx (double xi,double eta,vector &nodval)
{
  double f;
  vector bf(nne);
  
  bf_quad_3_2d (bf.a,xi,eta);
  scprd (bf,nodval,f);
  return f;
}

/**
   function returns matrix of base function

   17.8.2001
*/
void planeelemsubqt::bf_matrix (matrix &n,double xi,double eta)
{
  long i,i1,i2;
  vector bf(nne);
  
  bf_quad_3_2d (bf.a,xi,eta);
  fillm (0.0,n);
  
  i1=0;  i2=1;
  for (i=0;i<nne;i++){
    n[0][i1]=bf[i];  i1+=2;
    n[1][i2]=bf[i];  i2+=2;
  }
}

/**
   function assembles geometric matrix
   
   @param gm - geometric matrix
   @param x,y - node coordinates
   @param xi,eta - natural coordinates
   
*/
void planeelemsubqt::geom_matrix (matrix &gm,vector &x,vector &y,double xi,double eta)
{
  long i,i1,i2;
  double jac;
  vector dx(nne),dy(nne);

  dx_bf_quad_3_2d (dx.a,xi,eta);
  dy_bf_quad_3_2d (dy.a,xi,eta);

  derivatives_2d (dx,dy,jac,x,y,xi,eta);

  fillm (0.0,gm);

  i1=0;  i2=1;
  for (i=0;i<nne;i++){
    gm[0][i1]=dx[i];
    gm[1][i2]=dy[i];
    gm[2][i1]=dy[i];  i1+=2;
    gm[2][i2]=dx[i];  i2+=2;
  }
}

/**
   function assembles blocks of geometric matrix
   
   @param gm - geometric matrix
   @param x,y - node coordinates
   @param ri - row index
   @param xi,eta - natural coordinates
   
*/
void planeelemsubqt::geom_matrix_block (matrix &gm,double ri,vector &x,vector &y,double xi,double eta)
{
  long i,i1,i2;
  double jac;
  vector dx(nne),dy(nne);
  
  dx_bf_quad_3_2d (dx.a,xi,eta);
  dy_bf_quad_3_2d (dy.a,xi,eta);

  derivatives_2d (dx,dy,jac,x,y,xi,eta);

  fillm (0.0,gm);

  if (ri==0){
    i1=0;  i2=1;
    for (i=0;i<nne;i++){
      gm[0][i1]=dx[i];  i1+=2;
      gm[1][i2]=dy[i];  i2+=2;
    }
  }
  
  if (ri==1){
    i1=0;  i2=1;
    for (i=0;i<nne;i++){
      gm[0][i1]=dy[i];  i1+=2;
      gm[0][i2]=dx[i];  i2+=2;
    }
  }
}

void planeelemsubqt::dmatblock (long ri,long ci,matrix &d, matrix &dd)
{
  fillm (0.0,dd);
  
  if (ri==0 && ci==0){
    dd[0][0]=d[0][0];  dd[0][1]=d[0][1];
    dd[1][0]=d[1][0];  dd[1][1]=d[1][1];
  }
  if (ri==0 && ci==1){
    dd[0][0]=d[0][2];
    dd[1][0]=d[1][2];
  }
  if (ri==1 && ci==0){
    dd[0][0]=d[2][0];  dd[0][1]=d[2][1];
  }
  if (ri==1 && ci==1){
    dd[0][0]=d[2][2];
  }
}

/**
   transformation matrix x_g = T x_l

   17.8.2001
*/
void planeelemsubqt::transf_matrix (ivector &nodes,matrix &tmat)
{
  long i,n,m;

  fillm (0.0,tmat);

  n=nodes.n;
  m=tmat.m;
  for (i=0;i<m;i++){
    tmat[i][i]=1.0;
  }
  
  for (i=0;i<n;i++){
    if (Mt->nodes[nodes[i]].transf>0){
      tmat[i*2][i*2]   = Mt->nodes[nodes[i]].e1[0];  tmat[i*2][i*2+1]   = Mt->nodes[nodes[i]].e2[0];
      tmat[i*2+1][i*2] = Mt->nodes[nodes[i]].e1[1];  tmat[i*2+1][i*2+1] = Mt->nodes[nodes[i]].e2[1];
    }
  }
}

/**
   function computes stiffness matrix of plane stress triangular
   finite element with quadratic approximation functions

   @param eid - number of element
   @param sm - stiffness matrix

   25.8.2001
*/
void planeelemsubqt::stiffness_matrix (long eid,long ri,long ci,matrix &sm,vector &x,vector &y)
{
  long i,ii,jj,ipp;
  double jac,det,thick;
  ivector nodes(nne);
  vector t(nne),gp1,gp2,w;
  matrix gmr,gmc,dd,d(tncomp,tncomp);
  
  Mt->give_elemnodes (eid,nodes);
  Mc->give_thickness (eid,nodes,t);
  
  
  //  det is equal to double area of the element
  det = (x[1]-x[0])*(y[2]-y[0])-(x[2]-x[0])*(y[1]-y[0]);
  
  fillm (0.0,sm);
  
  for (ii=0;ii<nb;ii++){
    allocm (ncomp[ii],ndofe,gmr);
    for (jj=0;jj<nb;jj++){
      if (intordsm[ii][jj]==0)  continue;
      
      allocm (ncomp[jj],ndofe,gmc);
      allocm (ncomp[ii],ncomp[jj],dd);
      allocv (intordsm[ii][jj],gp1);
      allocv (intordsm[ii][jj],gp2);
      allocv (intordsm[ii][jj],w);
      
      gauss_points_tr (gp1.a,gp2.a,w.a,intordsm[ii][jj]);
      ipp=Mt->elements[eid].ipp[ri+ii][ci+jj];
      
      for (i=0;i<intordsm[ii][jj];i++){
	// geometric matrix
	geom_matrix_block (gmr,ii,x,y,gp1[i],gp2[i]);
	geom_matrix_block (gmc,jj,x,y,gp1[i],gp2[i]);
	
	//  stiffness matrix of material
	Mm->matstiff (d,ipp);  ipp++;
	dmatblock (ii,jj,d,dd);
	
	thick = approx (gp1[i],gp2[i],t);
	
	//  det is equal to double area of the element
	jac=w[i]*thick*det;
    
	//  contribution to the stiffness matrix of the element
	bdbjac (sm,gmr,dd,gmc,jac);
      }
      destrm (dd);  destrm (gmc);  destrv (gp1);  destrv (gp2);  destrv (w);
    }
    destrm (gmr);
  }
  
}

void planeelemsubqt::res_stiffness_matrix (long eid,matrix &sm)
{
  long transf;
  ivector nodes(nne);
  vector x(nne),y(nne);
  Mt->give_node_coord2d (x,y,eid);
  Mt->give_elemnodes (eid,nodes);

  stiffness_matrix (eid,0,0,sm,x,y);

  //  transformation of stiffness matrix
  transf = Mt->locsystems (nodes);
  if (transf>0){
    matrix tmat (ndofe,ndofe);
    transf_matrix (nodes,tmat);
    glmatrixtransf (sm,tmat);
  }
}

/**
   function computes mass matrix of the plane stress triangular
   finite element with quadratic approximation functions

   @param eid - number of element
   @param mm - mass matrix

   25.8.2001
*/
void planeelemsubqt::mass_matrix (long eid,matrix &mm,vector &x,vector &y)
{
  long i;
  double jac,det,thick,rho;
  ivector nodes(nne);
  vector w(intordmm),gp1(intordmm),gp2(intordmm),t(nne),dens(nne);
  matrix n(napfun,ndofe);
  
  Mt->give_elemnodes (eid,nodes);
  Mc->give_thickness (eid,nodes,t);
  Mc->give_density (eid,nodes,dens);

  //  det is equal to double area of the element
  det = (x[1]-x[0])*(y[2]-y[0])-(x[2]-x[0])*(y[1]-y[0]);
  
  gauss_points_tr (gp1.a,gp2.a,w.a,intordmm);
  
  fillm (0.0,mm);
  
  for (i=0;i<intordmm;i++){
    bf_matrix (n,gp1[i],gp2[i]);
    
    thick = approx (gp1[i],gp2[i],t);
    rho = approx (gp1[i],gp2[i],dens);
    
    jac=w[i]*thick*rho*det;
    
    nnj (mm.a,n.a,jac,n.m,n.n);
  }
  
}

void planeelemsubqt::res_mass_matrix (long eid,matrix &mm)
{
  vector x(nne),y(nne);
  Mt->give_node_coord2d (x,y,eid);
  mass_matrix (eid,mm,x,y);
}


void planeelemsubqt::load_matrix (long eid,matrix &lm)
  //  function computes load matrix of the plane stress triangular
  //  finite element with quadratic approximation functions
  //  load vector is obtained after premultiplying load matrix
  //  by nodal load values
  //  
  //  eid - number of element
  //  lm - load matrix
  //
  //  25.8.2001
{
  long i;
  double jac,det,thick;
  ivector nodes(nne);
  vector x(nne),y(nne),w(intordmm),gp1(intordmm),gp2(intordmm),t(nne);
  matrix n(napfun,ndofe);
  
  Mt->give_elemnodes (eid,nodes);
  Mt->give_node_coord2d (x,y,eid);
  Mc->give_thickness (eid,nodes,t);

  gauss_points_tr (gp1.a,gp2.a,w.a,intordmm);

  //  det is equal to double area of the element
  det = (x[1]-x[0])*(y[2]-y[0])-(x[2]-x[0])*(y[1]-y[0]);
  
  fillm (0.0,lm);
  
  for (i=0;i<intordmm;i++){
    bf_matrix (n,gp1[i],gp2[i]);
    
    thick = approx (gp1[i],gp2[i],t);
    
    jac=w[i]*thick*det;
    
    nnj (lm.a,n.a,jac,n.m,n.n);
  }
  
}


void planeelemsubqt::res_mainip_strains (long lcid,long eid)
{
  vector x(nne),y(nne);
  Mt->give_node_coord2d (x,y,eid);
  mainip_strains (lcid,eid,0,0,x,y);
}


/**
   function computes strains in main integration points of element
   
   @param lcid - load case id
   @param eid - element id
   @param ri - row index
   @param ci - column index
   
   10.5.2002
*/
void planeelemsubqt::mainip_strains (long lcid,long eid,long ri,long ci,vector &x,vector &y)
{
  long i,ii,ipp;
  vector r(ndofe),gp1,gp2,w,eps,aux;
  ivector nodes(nne),cn(ndofe);
  matrix gm,tmat;

  Mt->give_elemnodes (eid,nodes);
  Mt->give_code_numbers (eid,cn.a);
  eldispl (lcid,eid,r.a,cn.a,ndofe);
  
  //  transformation of displacement vector
  long transf = Mt->locsystems (nodes);
  if (transf>0){
    allocv (ndofe,aux);
    allocm (ndofe,ndofe,tmat);
    transf_matrix (nodes,tmat);
    //locglobtransf (aux,r,tmat);
    lgvectortransf (aux,r,tmat);
    copyv (aux,r);
    destrv (aux);
    destrm (tmat);
  }

  
  for (ii=0;ii<nb;ii++){
    allocv (intordsm[ii][ii],gp1);
    allocv (intordsm[ii][ii],gp2);
    allocv (intordsm[ii][ii],w);
    allocv (ncomp[ii],eps);
    allocm (ncomp[ii],ndofe,gm);
    
    gauss_points_tr (gp1.a,gp2.a,w.a,intordsm[ii][ii]);
    
    ipp=Mt->elements[eid].ipp[ri+ii][ci+ii];
    for (i=0;i<intordsm[ii][ii];i++){
      geom_matrix_block (gm,ii,x,y,gp1[i],gp2[i]);
      mxv (gm,r,eps);
      
      Mm->storestrain (lcid,ipp,cncomp[ii],ncomp[ii],eps);
      ipp++;
    }
    destrm (gm);  destrv (eps);  destrv (w);  destrv (gp1);  destrv (gp2);
  }
  
}

/**
   function computes strains in nodes of element
   
   @param lcid - load case id
   @param eid - element id
   
   10.5.2002
*/
void planeelemsubqt::nod_strains (long lcid,long eid,long ri,long ci)
{
  long i,ii,ipp;
  double *lsm,*lhs,*rhs;
  vector nxi(nne),neta(nne),r(ndofe),gp1,gp2,w,eps,aux,natcoord(2);
  ivector nodes(nne);

  lsm = new double [9];

  nodecoord (nxi,neta);
  Mt->give_elemnodes (eid,nodes);

  for (ii=0;ii<nb;ii++){
    allocv (intordsm[ii][ii],gp1);
    allocv (intordsm[ii][ii],gp2);
    allocv (intordsm[ii][ii],w);
    allocv (ncomp[ii],eps);
    lhs = new double [ncomp[ii]*3];
    rhs = new double [ncomp[ii]*3];
    gauss_points_tr (gp1.a,gp2.a,w.a,intordsm[ii][ii]);
    
    nullv (lsm,9);
    nullv (rhs,ncomp[ii]*3);
    
    ipp=Mt->elements[eid].ipp[ri+ii][ci+ii];
    for (i=0;i<intordsm[ii][ii];i++){
      Mm->givestrain (lcid,ipp,cncomp[ii],ncomp[ii],eps);
      
      natcoord[0]=gp1[i];  natcoord[1]=gp2[i];
      matassem_lsm (lsm,natcoord);
      rhsassem_lsm (rhs,natcoord,eps);
      
      ipp++;
    }
    
    solve_lsm (lsm,lhs,rhs,Mp->zero,3,ncomp[ii]);
    Mt->strain_nodal_values (nodes,nxi,neta,nxi,lhs,2,cncomp[ii],ncomp[ii],lcid);

    delete [] lhs;  delete [] rhs;
    destrv (eps);  destrv (w);  destrv (gp1);  destrv (gp2);
  }
  
  delete [] lsm;
}

/**
   function computes strains on element
   
   @param val - array containing strains on element
   @param lcid - load case id
   @param eid - element id
   
   15.7.2002
*/
void planeelemsubqt::elem_strains (double **stra,long lcid,long eid,long ri,long ci)
{
  long i,ii,ipp;
  double xi,eta,*lsm,*lhs,*rhs;
  vector nxi(nne),neta(nne),gp1,gp2,w,eps,aux,natcoord(2);
  ivector nodes(nne);

  lsm = new double [9];
  
  nodecoord (nxi,neta);
  Mt->give_elemnodes (eid,nodes);
  
  for (ii=0;ii<nb;ii++){
    allocv (intordsm[ii][ii],gp1);
    allocv (intordsm[ii][ii],gp2);
    allocv (intordsm[ii][ii],w);
    allocv (ncomp[ii],eps);
    lhs = new double [ncomp[ii]*3];
    rhs = new double [ncomp[ii]*3];
    gauss_points_tr (gp1.a,gp2.a,w.a,intordsm[ii][ii]);
    
    nullv (lsm,9);
    nullv (rhs,ncomp[ii]*3);
    
    ipp=Mt->elements[eid].ipp[ri+ii][ci+ii];
    for (i=0;i<intordsm[ii][ii];i++){
      xi=gp1[i];
      eta=gp2[i];
      
      Mm->givestrain (lcid,ipp,cncomp[ii],ncomp[ii],eps);
      
      natcoord[0]=xi;  natcoord[1]=eta;
      matassem_lsm (lsm,natcoord);
      rhsassem_lsm (rhs,natcoord,eps);

      ipp++;
    }
    
    solve_lsm (lsm,lhs,rhs,Mp->zero,3,ncomp[ii]);
    nodal_values (stra,nxi,neta,nxi,lhs,2,cncomp[ii],ncomp[ii]);

    delete [] lhs;  delete [] rhs;
    destrv (eps);  destrv (w);  destrv (gp1);  destrv (gp2);
  }
  
  delete [] lsm;
}


/**
   function computes strains in arbitrary point on element
   
   @param lcid - load case id
   @param eid - element id
   @param xi,eta - natural coordinates
   @param fi,li - first and last indices
   @param eps - array containing strains
   
   11.5.2002
*/
void planeelemsubqt::appstrain (long lcid,long eid,double xi,double eta,long fi,long ncomp,vector &eps)
{
  long i,j,k;
  ivector nodes;
  vector nodval;
  
  if (ncomp != eps.n){
    fprintf (stderr,"\n\n wrong interval of indices in function strain (%s, line %d).\n",__FILE__,__LINE__);
    abort ();
  }

  allocv (nne,nodes);
  allocv (nne,nodval);
  Mt->give_elemnodes (eid,nodes);
  k=0;
  for (i=fi;i<fi+ncomp;i++){
    for (j=0;j<nne;j++){
      nodval[j]=Mt->nodes[nodes[j]].strain[lcid*tncomp+i];
    }
    eps[k]=approx (xi,eta,nodval);
    k++;
  }
  
  destrv (nodes);  destrv (nodval);
}

/**
   function computes strains in all integration points
   
   @param lcid - load case id
   @param eid - element id
   @param ri,ci - row and column indices
   
   10.5.2002
*/
void planeelemsubqt::allip_strains (double **stra,long lcid,long eid,long ri,long ci)
{
  long i,ii,jj,ipp;
  //double xi,eta;
  vector eps(tncomp),gp1,gp2,w;
  
  for (ii=0;ii<nb;ii++){
    for (jj=0;jj<nb;jj++){
      if (intordsm[ii][jj]==0)  continue;

      allocv (intordsm[ii][jj],gp1);
      allocv (intordsm[ii][jj],gp2);
      allocv (intordsm[ii][jj],w);
      gauss_points_tr (gp1.a,gp2.a,w.a,intordsm[ii][jj]);
      ipp=Mt->elements[eid].ipp[ri+ii][ci+jj];
      
      for (i=0;i<intordsm[ii][jj];i++){
	if (Mp->strainaver==0)
	  //appval (xi,eta,0,tncomp,eps,stra);
	  appval (gp1[i],gp2[i],0,tncomp,eps,stra);
	if (Mp->strainaver==1)
	  appstrain (lcid,eid,gp1[i],gp2[i],0,tncomp,eps);

	Mm->storestrain (lcid,ipp,eps);
	ipp++;
      }
      destrv (w);  destrv (gp2);  destrv (gp1);
    }
  }
}

void planeelemsubqt::strains (long lcid,long eid,long ri,long ci)
{
  long i,naep,ncp,sid;
  double **stra;
  vector coord,eps;
  
  if (Mp->strainaver==0){
    stra = new double* [nne];
    for (i=0;i<nne;i++){
      stra[i] = new double [tncomp];
    }
    elem_strains (stra,lcid,eid,ri,ci);
  }

  switch (Mm->stra.tape[eid]){
  case nowhere:{
    break;
  }
  case intpts:{
    allip_strains (stra,lcid,eid,ri,ci);
    break;
  }
  case enodes:{
    break;
  }
  case userdefined:{
    //  number of auxiliary element points
    naep = Mm->stra.give_naep (eid);
    ncp = Mm->stra.give_ncomp (eid);
    sid = Mm->stra.give_sid (eid);
    allocv (ncp,eps);
    allocv (2,coord);
    for (i=0;i<naep;i++){
      Mm->stra.give_aepcoord (sid,i,coord);
      
      if (Mp->strainaver==0)
	appval (coord[0],coord[1],0,ncp,eps,stra);
      if (Mp->strainaver==1)
	appstrain (lcid,eid,coord[0],coord[1],0,ncp,eps);

      Mm->stra.storevalues(lcid,eid,i,eps);
    }
    destrv (eps);
    destrv (coord);
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown strain point is required in function planeelemlt::strains (%s, line %d).\n",__FILE__,__LINE__);
  }
  }

  if (Mp->strainaver==0){
    for (i=0;i<nne;i++){
      delete [] stra[i];
    }
    delete [] stra;
  }
}

/**
   function assembles natural coordinates of nodes of element
   
   @param xi - array containing natural coordinates xi
   @param eta - array containing natrual coordinates eta
   
   10.5.2002
*/
void planeelemsubqt::nodecoord (vector &xi,vector &eta)
{
  xi[0] = 1.0;  eta[0] = 0.0;
  xi[1] = 0.0;  eta[1] = 1.0;
  xi[2] = 0.0;  eta[2] = 0.0;
  xi[3] = 0.5;  eta[3] = 0.5;
  xi[4] = 0.0;  eta[4] = 0.5;
  xi[5] = 0.5;  eta[5] = 0.0;
}


/**
   function computes strains in arbitrary point on element
   
   @param xi, eta - natural coordinates of the point
   @param eps - array containing strains
   @param val - array containing values on element
   
   11.5.2002
*/
void planeelemsubqt::appval (double xi,double eta,long fi,long nc,vector &eps,double **val)
{
  long i,j,k;
  vector nodval;
  
  k=0;
  allocv (nne,nodval);
  for (i=fi;i<fi+nc;i++){
    for (j=0;j<nne;j++){
      nodval[j]=val[j][i];
    }
    eps[k]=approx (xi,eta,nodval);
    k++;
  }
  
  destrv (nodval);
}

/**
   function computes stresses in main integration points of element
   
   @param lcid - load case id
   @param eid - element id
   @param ri - row index
   @param ci - column index
   
   10.5.2002
*/
void planeelemsubqt::mainip_stresses (long lcid,long eid,long ri,long ci)
{
  long i,ii,jj,ipp;
  vector gp1,gp2,w,eps,sig,auxsig;
  matrix d(tncomp,tncomp),dd;

  for (ii=0;ii<nb;ii++){
    allocv (ncomp[ii],sig);
    allocv (ncomp[ii],auxsig);
    allocv (intordsm[ii][ii],gp1);
    allocv (intordsm[ii][ii],gp2);
    allocv (intordsm[ii][ii],w);
    
    gauss_points_tr (gp1.a,gp2.a,w.a,intordsm[ii][ii]);
    ipp=Mt->elements[eid].ipp[ri+ii][ci+ii];
    
    for (i=0;i<intordsm[ii][ii];i++){
      Mm->matstiff (d,ipp);
      
      fillv (0.0,sig);
      for (jj=0;jj<nb;jj++){
	allocv (ncomp[jj],eps);
	allocm (ncomp[ii],ncomp[jj],dd);
	
	if (Mp->strainaver==0)
	  Mm->givestrain (lcid,ipp,cncomp[jj],ncomp[jj],eps);
	if (Mp->strainaver==1)
	  appstrain (lcid,eid,gp1[i],gp2[i],cncomp[jj],ncomp[jj],eps);
	
	dmatblock (ii,jj,d,dd);
	mxv (dd,eps,auxsig);
	addv (auxsig,sig,sig);
	destrm (dd);  destrv (eps);
      }
      
      Mm->storestress (lcid,ipp,sig);
      
      ipp++;
    }
    destrv (w);  destrv (gp2);  destrv (gp1);  destrv (auxsig);  destrv (sig);
  }
}

/**
   function computes stresses in nodes
   
   @param lcid - load case id
   @param eid - element id
   @param ri,ci - row and column indices
   
   10.5.2002
*/
void planeelemsubqt::nod_stresses (long lcid,long eid,long ri,long ci)
{
  long i,ii,jj,ipp;
  double *lsm,*lhs,*rhs;
  vector nxi(nne),neta(nne),gp1,gp2,w,eps,sig,auxsig,natcoord(2);
  ivector nodes(nne);
  matrix d(tncomp,tncomp),dd;
  
  lsm = new double [9];

  Mt->give_elemnodes (eid,nodes);
  
  nodecoord (nxi,neta);

  for (ii=0;ii<nb;ii++){
    allocv (intordsm[ii][ii],gp1);
    allocv (intordsm[ii][ii],gp2);
    allocv (intordsm[ii][ii],w);
    allocv (ncomp[ii],sig);
    allocv (ncomp[ii],auxsig);
    lhs = new double [ncomp[ii]*3];
    rhs = new double [ncomp[ii]*3];
    gauss_points_tr (gp1.a,gp2.a,w.a,intordsm[ii][ii]);
    
    nullv (lsm,9);
    nullv (rhs,ncomp[ii]*3);
    
    ipp=Mt->elements[eid].ipp[ri+ii][ci+ii];
    
    for (i=0;i<intordsm[ii][ii];i++){
      
      Mm->matstiff (d,ipp);
      ipp++;
      
      fillv (0.0,sig);
      for (jj=0;jj<nb;jj++){
	allocv (ncomp[jj],eps);
	allocm (ncomp[ii],ncomp[jj],dd);
	
	if (Mp->strainaver==0)
	  Mm->givestrain (lcid,ipp,cncomp[jj],ncomp[jj],eps);
	if (Mp->strainaver==1)
	  appstrain (lcid,eid,gp1[i],gp2[i],cncomp[jj],ncomp[jj],eps);
	
	dmatblock (ii,jj,d,dd);
	mxv (dd,eps,auxsig);
	addv (auxsig,sig,sig);
	destrm (dd);  destrv (eps);
      }
      
      natcoord[0]=gp1[i];  natcoord[1]=gp2[i];
      matassem_lsm (lsm,natcoord);
      rhsassem_lsm (rhs,natcoord,sig);
      
    }
    
    solve_lsm (lsm,lhs,rhs,Mp->zero,3,ncomp[ii]);
    Mt->stress_nodal_values (nodes,nxi,neta,nxi,lhs,2,cncomp[ii],ncomp[ii],lcid);
    
    
    delete [] lhs;  delete [] rhs;
    destrv (auxsig);  destrv (sig);  destrv (eps);  destrv (w);  destrv (gp2);  destrv (gp1);
  }
  
  delete [] lsm;
}


void planeelemsubqt::elem_stresses (double **stra,double **stre,long lcid,long eid,long ri,long ci)
{
  long i,ii,jj,ipp;
  double *lsm,*lhs,*rhs;
  vector nxi(nne),neta(nne),gp1,gp2,w,eps,sig,auxsig,natcoord(2);
  ivector nodes(nne);
  matrix d(tncomp,tncomp),dd;
  
  lsm = new double [9];

  Mt->give_elemnodes (eid,nodes);
  
  nodecoord (nxi,neta);

  for (ii=0;ii<nb;ii++){
    allocv (intordsm[ii][ii],gp1);
    allocv (intordsm[ii][ii],gp2);
    allocv (intordsm[ii][ii],w);
    allocv (ncomp[ii],sig);
    allocv (ncomp[ii],auxsig);
    lhs = new double [ncomp[ii]*3];
    rhs = new double [ncomp[ii]*3];
    gauss_points_tr (gp1.a,gp2.a,w.a,intordsm[ii][ii]);
    
    nullv (lsm,9);
    nullv (rhs,ncomp[ii]*3);
    
    ipp=Mt->elements[eid].ipp[ri+ii][ci+ii];
    
    for (i=0;i<intordsm[ii][ii];i++){
      
      Mm->matstiff (d,ipp);
      ipp++;
      
      fillv (0.0,sig);
      for (jj=0;jj<nb;jj++){
	allocv (ncomp[jj],eps);
	allocm (ncomp[ii],ncomp[jj],dd);

	if (Mp->strainaver==0)
	  appval (gp1[i],gp2[i],cncomp[jj],ncomp[jj],eps,stra);
	if (Mp->strainaver==1)
	  appstrain (lcid,eid,gp1[i],gp2[i],cncomp[jj],ncomp[jj],eps);
	
	dmatblock (ii,jj,d,dd);
	mxv (dd,eps,auxsig);
	addv (auxsig,sig,sig);
	destrm (dd);  destrv (eps);
      }
      
      natcoord[0]=gp1[i];  natcoord[1]=gp2[i];
      matassem_lsm (lsm,natcoord);
      rhsassem_lsm (rhs,natcoord,sig);
      
    }
    
    solve_lsm (lsm,lhs,rhs,Mp->zero,3,ncomp[ii]);
    nodal_values (stre,nxi,neta,nxi,lhs,2,cncomp[ii],ncomp[ii]);
    
    delete [] lhs;  delete [] rhs;
    destrv (auxsig);  destrv (sig);  destrv (eps);  destrv (w);  destrv (gp2);  destrv (gp1);
  }
  
  delete [] lsm;
}


/**
   function computes stresses in arbitrary point on element
   
   @param lcid - load case id
   @param eid - element id
   @param xi,eta - natural coordinates
   @param fi,li - first and last indices
   @param sig - array containing stresses
   
   11.5.2002
*/
void planeelemsubqt::appstress (long lcid,long eid,double xi,double eta,long fi,long ncomp,vector &sig)
{
  long i,j,k;
  ivector nodes;
  vector nodval;
  
  if (ncomp != sig.n){
    fprintf (stderr,"\n\n wrong interval of indices in function stress (%s, line %d).\n",__FILE__,__LINE__);
    abort ();
  }

  allocv (nne,nodes);
  allocv (nne,nodval);
  
  Mt->give_elemnodes (eid,nodes);
  k=0;
  for (i=fi;i<fi+ncomp;i++){
    for (j=0;j<nne;j++){
      nodval[j]=Mt->nodes[nodes[j]].stress[lcid*tncomp+i];
    }
    sig[k]=approx (xi,eta,nodval);
    k++;
  }
  
  destrv (nodes);  destrv (nodval);
}

/**
   function computes stresses in all integration points
   
   @param lcid - load case id
   @param eid - element id
   @param ri,ci - row and column indices
   
   10.5.2002
*/
void planeelemsubqt::allip_stresses (double **stre,long lcid,long eid,long ri,long ci)
{
  long i,ii,jj,ipp;
  vector sig,gp1,gp2,w;
  
  allocv (tncomp,sig);

  for (ii=0;ii<nb;ii++){
    for (jj=0;jj<nb;jj++){
      if (intordsm[ii][jj]==0)  continue;

      allocv (intordsm[ii][jj],gp1);
      allocv (intordsm[ii][jj],gp2);
      allocv (intordsm[ii][jj],w);
      gauss_points_tr (gp1.a,gp2.a,w.a,intordsm[ii][jj]);
      ipp=Mt->elements[eid].ipp[ri+ii][ci+jj];
      
      for (i=0;i<intordsm[ii][jj];i++){
	appstress (lcid,eid,gp1[i],gp2[i],0,tncomp,sig);
	Mm->storestress (lcid,ipp,sig);
	ipp++;
      }
      destrv (w);  destrv (gp2);  destrv (gp1);
    }
  }
  destrv (sig);
}

void planeelemsubqt::stresses (long lcid,long eid,long ri,long ci)
{
  long i,naep,ncp,sid;
  double **stra,**stre;
  vector coord,sig;
  
  if (Mp->stressaver==0){
    stra = new double* [nne];
    stre = new double* [nne];
    for (i=0;i<nne;i++){
      stra[i] = new double [tncomp];
      stre[i] = new double [tncomp];
    }
    elem_strains (stra,lcid,eid,ri,ci);
    elem_stresses (stra,stre,lcid,eid,ri,ci);
  }

  switch (Mm->stre.tape[eid]){
  case nowhere:{
    break;
  }
  case intpts:{
    allip_stresses (stre,lcid,eid,ri,ci);
    break;
  }
  case enodes:{
    break;
  }
  case userdefined:{
    //  number of auxiliary element points
    naep = Mm->stre.give_naep (eid);
    ncp = Mm->stre.give_ncomp (eid);
    sid = Mm->stre.give_sid (eid);
    allocv (ncp,sig);
    allocv (2,coord);
    for (i=0;i<naep;i++){
      Mm->stre.give_aepcoord (sid,i,coord);
      
      if (Mp->stressaver==0)
	appval (coord[0],coord[1],0,ncp,sig,stre);
      if (Mp->stressaver==1)
	appstress (lcid,eid,coord[0],coord[1],0,ncp,sig);
      
      Mm->stre.storevalues(lcid,eid,i,sig);
    }
    destrv (sig);
    destrv (coord);
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown stress point is required in function planeelemlq::stresses (%s, line %d).\n",__FILE__,__LINE__);
  }
  }

  if (Mp->stressaver==0){
    for (i=0;i<nne;i++){
      delete [] stra[i];
      delete [] stre[i];
    }
    delete [] stra;
    delete [] stre;
  }
}






/**
   function computes internal forces

   @param lcid - load case id
   @param eid - element id
   @param ri,ci - row and column indices
   @param ifor - vector of internal forces
   @param x,y - vectors of nodal coordinates

   TKo 7.2008
*/
void planeelemsubqt::internal_forces (long lcid,long eid,long ri,long ci,vector &ifor,vector &x,vector &y)
{
  integratedquant iq;
  iq=locstress;

  //  computation of stresses
  compute_nlstress (lcid,eid,ri,ci);
  //  integration of stresses over the element
  elem_integration (iq,lcid,eid,ri,ci,ifor,x,y);
}



/**
   function computes internal forces for nonlocal models

   @param lcid - load case id
   @param eid - element id
   @param ri,ci - row and column indices
   @param ifor - vector of internal forces
   @param x,y - vectors of nodal coordinates

   TKo 7.2008
*/
void planeelemsubqt::nonloc_internal_forces (long lcid,long eid,long ri,long ci,vector &ifor,vector &x,vector &y)
{
  integratedquant iq;
  iq=nonlocstress;

  //  computation of stresses
  compute_nonloc_nlstress (lcid,eid,ri,ci);
  //  integration of stresses over the element
  elem_integration (iq,lcid,eid,ri,ci,ifor,x,y);
}



/**
   function computes increments of internal forces

   @param lcid - load case id
   @param eid - element id
   @param ri,ci - row and column indices
   @param ifor - vector of internal forces
   @param x,y - vectors of nodal coordinates

   TKo 7.2008
*/
void planeelemsubqt::incr_internal_forces (long lcid,long eid,long ri,long ci,vector &ifor,vector &x,vector &y)
{
  integratedquant iq;
  iq=stressincr;

  //  computation of stresses
  compute_nlstressincr (lcid,eid,ri,ci);
  //  integration of stresses over the element
  elem_integration (iq,lcid,eid,ri,ci,ifor,x,y);
}



/**
   function computes nodal forces caused by temperature changes
   
   @param eid - element id
   @param ri,ci - row and column indices
   @param nfor - array containing nodal forces
   @param x,y - nodal coordinates
   
   7.2008, TKo
*/
void planeelemsubqt::eigstrain_forces (long lcid,long eid,long ri,long ci,vector &nfor,vector &x,vector &y)
{
  integratedquant iq;
  iq=eigstress;
  
  //  computation of eigenstresses
  compute_eigstress (lcid,eid,ri,ci);
  
  //  integration of stresses over the element
  elem_integration (iq,lcid,eid,ri,ci,nfor,x,y);
}



/**
   function computes resulting internal forces
   
   @param lcid - load case id
   @param eid - element id
   @param ifor - %vector of internal forces
   
   TKo, 7.2008
*/
void planeelemsubqt::res_internal_forces (long lcid,long eid,vector &ifor)
{
  long transf;
  ivector nodes (nne);
  vector v(ndofe),x(nne),y(nne);
  
  Mt->give_node_coord2d (x,y,eid);
  internal_forces (lcid,eid,0,0,ifor,x,y);
  //  transformation of nodal forces
  //  (in the case of nodal coordinate systems)
  Mt->give_elemnodes (eid,nodes);
  transf = Mt->locsystems (nodes);
  if (transf>0){
    matrix tmat (ndofe,ndofe);
    transf_matrix (nodes,tmat);
    glvectortransf (ifor,v,tmat);
    copyv (v,ifor);
  }
}



/**
   function computes resulting internal forces for nonlocal models
   
   @param lcid - load case id
   @param eid - element id
   @param ifor - %vector of internal forces
   
   TKo, 7.2008
*/
void planeelemsubqt::res_nonloc_internal_forces (long lcid,long eid,vector &ifor)
{
  long transf;
  ivector nodes (nne);
  vector v(ndofe),x(nne),y(nne);
  
  Mt->give_node_coord2d (x,y,eid);
  nonloc_internal_forces (lcid,eid,0,0,ifor,x,y);
  //  transformation of nodal forces
  //  (in the case of nodal coordinate systems)
  Mt->give_elemnodes (eid,nodes);
  transf = Mt->locsystems (nodes);
  if (transf>0){
    matrix tmat (ndofe,ndofe);
    transf_matrix (nodes,tmat);
    glvectortransf (ifor,v,tmat);
    copyv (v,ifor);
  }
}



/**
   function computes resulting increment of internal forces
   
   @param lcid - load case id
   @param eid - element id
   @param ifor - %vector of internal forces
   
   TKo, 7.2008
*/
void planeelemsubqt::res_incr_internal_forces (long lcid,long eid,vector &ifor)
{
  long transf;
  ivector nodes (nne);
  vector v(ndofe),x(nne),y(nne);
  
  Mt->give_node_coord2d (x,y,eid);
  incr_internal_forces (lcid,eid,0,0,ifor,x,y);
  //  transformation of nodal forces
  //  (in the case of nodal coordinate systems)
  Mt->give_elemnodes (eid,nodes);
  transf = Mt->locsystems (nodes);
  if (transf>0){
    matrix tmat (ndofe,ndofe);
    transf_matrix (nodes,tmat);
    glvectortransf (ifor,v,tmat);
    copyv (v,ifor);
  }
}



/**
   function computes resulting contributions from eigenstrains to the right hand side
   
   @param lcid - load case id
   @param eid - element id
   @param nfor - %vector of internal forces

   TKo, 7.2008
*/
void planeelemsubqt::res_eigstrain_forces (long lcid,long eid,vector &nfor)
{
  long transf;
  ivector nodes (nne);
  vector v(ndofe),x(nne),y(nne);
  
  Mt->give_node_coord2d (x,y,eid);
  eigstrain_forces (lcid,eid,0,0,nfor,x,y);
  //  transformation of nodal forces
  //  (in the case of nodal coordinate systems)
  Mt->give_elemnodes (eid,nodes);
  transf = Mt->locsystems (nodes);
  if (transf>0){
    matrix tmat (ndofe,ndofe);
    transf_matrix (nodes,tmat);
    glvectortransf (nfor,v,tmat);
    copyv (v,nfor);
  }
}



/**
   function computes correct stresses at integration points on element

   @param lcid - number of load case
   @param eid - element id
   @param ri,ci - row and column indices
   
   TKo 7.2008
*/
void planeelemsubqt::compute_nlstress (long lcid,long eid,long ri,long ci)
{
  long i,ii,ipp;
  
  for (ii=0;ii<nb;ii++){
    ipp=Mt->elements[eid].ipp[ri+ii][ci+ii];
    
    for (i=0;i<intordsm[ii][ii];i++){
      //  computation of correct stresses
      if (Mp->strcomp==1)
	Mm->computenlstresses (ipp);
      ipp++;
    }
  }
}


             
/**
   function computes local values which will be used for averageing 
   in nonlocal models at integration points. Mp->nonlocphase have to be 1.

   @param lcid - number of load case
   @param eid - element id
   @param ri,ci - row and column indices
   
   TKo, 7.2008
*/
void planeelemsubqt::local_values(long lcid,long eid,long ri,long ci)
{
  long i,ii,ipp;

  for (ii=0;ii<nb;ii++){
    ipp=Mt->elements[eid].ipp[ri+ii][ci+ii];
    
    for (i=0;i<intordsm[ii][ii];i++){
      //  computation of correct stresses
      if (Mp->strcomp==1)
	Mm->computenlstresses (ipp);
      ipp++;
    }
  }
}


     
/**
   function computes nonlocal correct stresses at integration points on element

   @param lcid - number of load case
   @param eid - element id
   @param ri,ci - row and column indices
   
   TKo, 7.2008
*/
void planeelemsubqt::compute_nonloc_nlstress(long lcid,long eid,long ri,long ci)
{
  long i,ii,ipp;
  
  for (ii=0;ii<nb;ii++){
    ipp=Mt->elements[eid].ipp[ri+ii][ci+ii];
    
    for (i=0;i<intordsm[ii][ii];i++){
      //  computation of correct stresses
      if (Mp->strcomp==1)
        Mm->compnonloc_nlstresses (ipp);
      ipp++;
    }
  }
}


     
/**
   function computes correct stress increments at integration points on element

   @param lcid - number of load case
   @param eid - element id
   @param ri,ci - row and column indices
   
   TKo, 7.2008
*/
void planeelemsubqt::compute_nlstressincr(long lcid,long eid,long ri,long ci)
{
  long i,ii,ipp;
  
  for (ii=0;ii<nb;ii++){
    ipp=Mt->elements[eid].ipp[ri+ii][ci+ii];
    
    for (i=0;i<intordsm[ii][ii];i++){
      //  computation of correct stresses
      if (Mp->strcomp==1)
        Mm->computenlstressesincr (ipp);
      ipp++;
    }
  }
}


     
/**
   function computes correct eigen stresses caused by temperature at integration points on element
   
   @param lcid - number of load case
   @param eid - element id
   @param ri,ci - row and column indices
   
   TKo 7.2008
*/
void planeelemsubqt::compute_eigstress(long lcid,long eid,long ri,long ci)
{
  long i,ii,ipp;
  vector eigstr(tncomp),sig(tncomp);
  matrix d(tncomp,tncomp);
  
  for (ii=0;ii<nb;ii++){
    ipp=Mt->elements[eid].ipp[ri+ii][ci+ii];
    
    for (i=0;i<intordsm[ii][ii];i++){
      Mm->giveeigstrain (ipp,eigstr);
      //  matrix of stiffness of the material
      Mm->matstiff (d,ipp);
      mxv (d,eigstr,sig);
      Mm->storeeigstress (ipp,sig);
      ipp++;
    }
  }
}


     
/**
   function integrates selected quantity over the finite element
   it results in nodal values
   
   @param iq - type of integrated quantity (see alias.h)
   @param lcid - number of load case
   @param eid - element id
   @param ri,ci - row and column indices
   @param nv - nodal values
   @param x,y - node coordinates
   
   TKo 7.2008
*/
void planeelemsubqt::elem_integration(integratedquant iq,long lcid,long eid,long ri,long ci,vector &nv,vector &x,vector &y)
{
  long i,ii,ipp;
  double jac,thick;
  ivector nodes(nne);
  vector w,gp1,gp2,t(nne);
  vector ipv,contr(ndofe);
  matrix gm;
  
  Mt->give_elemnodes (eid,nodes);
  Mc->give_thickness (eid,nodes,t);
  
  //  det is equal to double area of the element
  jac = (x[1]-x[0])*(y[2]-y[0])-(x[2]-x[0])*(y[1]-y[0]);

  fillv (0.0,nv);
  
  for (ii=0;ii<nb;ii++){
    allocv (intordsm[ii][ii],gp1);
    allocv (intordsm[ii][ii],gp2);
    allocv (intordsm[ii][ii],w);
    
    gauss_points_tr (gp1.a,gp2.a,w.a,intordsm[ii][ii]);
    ipp=Mt->elements[eid].ipp[ri+ii][ci+ii];
    
    
    for (i=0;i<intordsm[ii][ii];i++){
      thick = approx (gp1[i],gp2[i],t);
      //  function assembles required quantity at integration point
      Mm->givequantity (iq,lcid,ipp,cncomp[ii],ipv);
      //  strain-displacement (geometric) matrix
      geom_matrix_block (gm,ii,x,y,gp1[i],gp2[i]);
      //  contribution to the nodal values
      mtxv (gm,ipv,contr);
      //  contribution to the nodal values
      cmulv (jac*w[i]*thick,contr);
      //  summation
      addv(contr,nv,nv);
      ipp++;
      
    }
    destrv (w);  destrv (gp2);  destrv (gp1);
  }
}





/**
   function assembles coordinates of integration points
   
   @param eid - element id
   @param ipp - integration point pointer
   @param ri - row index
   @param ci - column index
   @param ipcoord - array containing coordinates of integration points
   
   TKo, 7.2008
*/
void planeelemsubqt::ipcoord (long eid,long ipp,long ri,long ci,vector &coord)
{
  long ii,jj,i,aipp;
  vector x(nne),y(nne);
  vector w,gp1,gp2;
  
  Mt->give_node_coord2d (x,y,eid);
  for (ii=0;ii<nb;ii++){
    for (jj=0;jj<nb;jj++){
      if (intordsm[ii][jj]==0)  continue;
      
      allocv (intordsm[ii][jj],gp1);
      allocv (intordsm[ii][jj],gp2);
      allocv (intordsm[ii][jj],w);
      gauss_points_tr (gp1.a,gp2.a,w.a,intordsm[ii][jj]);
      aipp=Mt->elements[eid].ipp[ri+ii][ci+jj];
      for (i=0;i<intordsm[ii][jj];i++){
        if (aipp==ipp){
          coord[0]=approx (gp1[i],gp2[i],x);
          coord[1]=approx (gp1[i],gp2[i],y);
          coord[2]=0.0;
        }
        aipp++;
      }
      destrv(gp1); destrv(gp2); destrv(w);
    }
  }
}



void planeelemsubqt::nodeforces (long eid,long *le,double *nv,vector &nf)
{
  long i;
  double xi,eta,jac;
  vector x(nne),y(nne),gp(intordb),w(intordb),av(ndofe),v(ndofe);
  matrix n(napfun,ndofe),am(ndofe,ndofe);

  Mt->give_node_coord2d (x,y,eid);
  gauss_points (gp.a,w.a,intordb);

  if (le[0]==1){
    fillm (0.0,am);
    for (i=0;i<intordb;i++){
      xi=(1.0-gp[i])/2.0;  eta=(1.0+gp[i])/2.0;
      bf_matrix (n,xi,eta);

      jac1d_2d (jac,x,y,gp[i],0);
      jac*=w[i];

      nnj (am.a,n.a,jac,n.m,n.n);
    }
    fillv (0.0,av);
    av[0]=nv[0]; av[1]=nv[1]; av[6]=nv[2]; av[7]=nv[3]; av[2]=nv[4]; av[3]=nv[5];
    mxv (am,av,v);  addv (nf,v,nf);
  }
  if (le[1]==1){
    fillm (0.0,am);
    for (i=0;i<intordb;i++){
      xi=0.0;  eta=(1.0-gp[i])/2.0;
      bf_matrix (n,xi,eta);

      jac1d_2d (jac,x,y,gp[i],1);
      jac*=w[i];

      nnj (am.a,n.a,jac,n.m,n.n);
    }
    fillv (0.0,av);
    av[2]=nv[6]; av[3]=nv[7]; av[8]=nv[8]; av[9]=nv[9]; av[4]=nv[10]; av[5]=nv[11];
    mxv (am,av,v);  addv (nf,v,nf);
  }
  if (le[2]==1){
    fillm (0.0,am);
    for (i=0;i<intordb;i++){
      xi=(1.0+gp[i])/2.0;  eta=0.0;
      bf_matrix (n,xi,eta);

      jac1d_2d (jac,x,y,gp[i],2);
      jac*=w[i];

      nnj (am.a,n.a,jac,n.m,n.n);
    }
    fillv (0.0,av);
    av[4]=nv[12]; av[5]=nv[13]; av[10]=nv[14]; av[11]=nv[15]; av[0]=nv[16]; av[1]=nv[17];
    mxv (am,av,v);  addv (nf,v,nf);
  }
}



void planeelemsubqt::inicipval(long eid, long ri, long ci, matrix &nodval, inictype *ictn)
{
  long i, j, k, ipp;
  long ii, jj, nv = nodval.n;
  long nstra;
  double xi, eta, ipval;
  vector w, gp1, gp2, anv(nne);

  nstra = 0;
  for (j = 0; j < nv; j++) // for all initial values
  {
    for(i = 0; i < nne; i++) // for all nodes on element
      anv[i] = nodval[i][j];
    for (ii = 0; ii < nb; ii++)
    {
      for (jj = 0; jj < nb; jj++)
      {
        ipp=Mt->elements[eid].ipp[ri+ii][ci+jj];
        if (intordsm[ii][jj] == 0)
          continue;
        allocv (intordsm[ii][jj],gp1);
        allocv (intordsm[ii][jj],gp2);
        allocv (intordsm[ii][jj],w);
        gauss_points_tr (gp1.a, gp2.a, w.a, intordsm[ii][jj]);
        for (k = 0; k < intordsm[ii][jj]; k++)
        {
          xi=gp1[k];
          eta=gp2[k];
          //  value in integration point
          ipval = approx (xi,eta,anv);
          if ((ictn[i] & inistrain) && (j < Mm->ip[ipp].ncompstr))
          {
            Mm->ip[ipp].strain[j] += ipval;
            ipp++;
            continue;
          }
          if ((ictn[i] & inistress) && (j < nstra + Mm->ip[ipp].ncompstr))
          {
            Mm->ip[ipp].stress[j] += ipval;
            ipp++;
            continue;
          }
          if ((ictn[i] & iniother) && (j < nv))
          {
            Mm->ip[ipp].other[j] += ipval;
            ipp++;
            continue;
          }
          ipp++;
        }
        destrv(gp1);  destrv (gp2);  destrv (w);
      }
    }
    if (ictn[i] & inistrain) nstra++;
  }
}

